@@ -18,12 +18,19 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
 
-/* DHE_DSS is not really working. It is used as a template */
+/* DHE_DSS is not really working. It is used as a template 
+ * (it may work BUT it does not check certificates)
+ */
+
+#if 0
 
 #include <defines.h>
 #include "gnutls_int.h"
+#include "gnutls_auth_int.h"
 #include "gnutls_errors.h"
 #include "gnutls_dh.h"
+#include "auth_dhe_dss.h"
+#include "gnutls_num.h"
 
 int gen_dhe_dss_server_kx( GNUTLS_KEY , opaque**);
 int gen_dhe_dss_client_kx( GNUTLS_KEY , opaque**);
@@ -55,15 +62,30 @@ MOD_AUTH_STRUCT dhe_dss_auth_struct = {
 int gen_dhe_dss_server_kx( GNUTLS_KEY key, opaque** data) {
 	GNUTLS_MPI x, X, g, p;
 	size_t n_X, n_g, n_p;
-	uint16 _n_X, _n_g, _n_p;
 	uint8 *data_p;
 	uint8 *data_g;
 	uint8 *data_X;
 	int ret = 0;
+	DHE_DSS_SERVER_CREDENTIALS * cred;
+
+	cred = _gnutls_get_kx_cred( key, GNUTLS_KX_DHE_DSS, NULL);
+	if (cred==NULL) {
+		bits = DEFAULT_BITS; /* default */
+	} else {
+		bits = cred->bits;
+	}
+
+	g = gnutls_get_dh_params(&p, bits);
+
+	key->auth_info = gnutls_malloc(sizeof(DHE_DSS_AUTH_INFO));
+	if (key->auth_info==NULL) return GNUTLS_E_MEMORY_ERROR;
+	((DHE_DSS_AUTH_INFO*)key->auth_info)->bits = gcry_mpi_get_nbits(p);
+	key->auth_info_size = sizeof(DHE_DSS_AUTH_INFO);
+
+	X = gnutls_calc_dh_secret(&x, g, p);
 
-	X = gnutls_calc_dh_secret(&x);
 	key->dh_secret = x;
-	g = gnutls_get_dh_params(&p);
+
 	gcry_mpi_print(GCRYMPI_FMT_USG, NULL, &n_g, g);
 	gcry_mpi_print(GCRYMPI_FMT_USG, NULL, &n_p, p);
 	gcry_mpi_print(GCRYMPI_FMT_USG, NULL, &n_X, X);
@@ -71,33 +93,21 @@ int gen_dhe_dss_server_kx( GNUTLS_KEY key, opaque** data) {
 	data_p = &(*data)[0];
 	gcry_mpi_print(GCRYMPI_FMT_USG, &data_p[2], &n_p, p);
 	gnutls_mpi_release(p);
-	_n_p = n_p;
-#ifndef WORDS_BIGENDIAN
-	_n_p = byteswap16(_n_p);
-	memmove(data_p, &_n_p, 2);
-#else
-	memmove(data_p, &_n_p, 2);
-#endif
+
+	WRITEuint16( n_p, data_p);
+
 	data_g = &data_p[2 + n_p];
 	gcry_mpi_print(GCRYMPI_FMT_USG, &data_g[2], &n_g, g);
 	gnutls_mpi_release(g);
-	_n_g = n_g;
-#ifndef WORDS_BIGENDIAN
-	_n_g = byteswap16(_n_g);
-	memmove(data_g, &_n_g, 2);
-#else
-	memmove(data_g, &_n_g, 2);
-#endif
+
+	WRITEuint16( n_g, data_g);
+
 	data_X = &data_g[2 + n_g];
 	gcry_mpi_print(GCRYMPI_FMT_USG, &data_X[2], &n_X, X);
 	gnutls_mpi_release(X);
-	_n_X = n_X;
-#ifndef WORDS_BIGENDIAN
-	_n_X = byteswap16(_n_X);
-	memmove(data_X, &_n_X, 2);
-#else
-	memmove(data_X, &_n_X, 2);
-#endif
+
+	WRITEuint16( n_X, data_X);
+
 	ret = n_p+n_g+n_X+6;
 
 	return ret;
@@ -106,7 +116,6 @@ int gen_dhe_dss_server_kx( GNUTLS_KEY key, opaque** data) {
 int gen_dhe_dss_client_kx( GNUTLS_KEY key, opaque** data) {
 GNUTLS_MPI x, X;
 size_t n_X;
-uint16 _n_X;
 
 	X =  _gnutls_calc_dh_secret(&x, key->client_g,
 		   key->client_p);
@@ -118,13 +127,8 @@ uint16 _n_X;
 	(*data)[0] = 1;	/* extern - explicit since we do not have
 				   certificate */
 	gnutls_mpi_release(X);
-	_n_X = n_X;
-#ifndef WORDS_BIGENDIAN
-	_n_X = byteswap16(_n_X);
-	memmove(&(*data)[0], &_n_X, 2);
-#else
-	memmove(&(*data)[0], &_n_X, 2);
-#endif
+	
+	WRITEuint16( n_X, &(*data)[0]);    
 	
 	/* calculate the key after calculating the message */
 	key->KEY = _gnutls_calc_dh_key(key->client_Y, x, key->client_p);
@@ -150,11 +154,8 @@ int proc_dhe_dss_server_kx( GNUTLS_KEY key, opaque* data, int data_size) {
 
 
 	i = 0;
-	memmove(&n_p, &data[i], 2);
+	n_p = READuint16( &data[i]);
 	i += 2;
-#ifndef WORDS_BIGENDIAN
-	n_p = byteswap16(n_p);
-#endif
 
 	data_p = &data[i];
 	i += n_p;
@@ -162,22 +163,19 @@ int proc_dhe_dss_server_kx( GNUTLS_KEY key, opaque* data, int data_size) {
 		gnutls_assert();
 		return GNUTLS_E_UNEXPECTED_PACKET_LENGTH;
 	}
-	memmove(&n_g, &data[i], 2);
-#ifndef WORDS_BIGENDIAN
-	n_g = byteswap16(n_g);
-#endif
+	
+	n_g = READuint16( &data[i]);
 	i += 2;
+
 	data_g = &data[i];
 	i += n_g;
 	if (i > data_size) {
 		gnutls_assert();
 		return GNUTLS_E_UNEXPECTED_PACKET_LENGTH;
 	}
-	memmove(&n_Y, &data[i], 2);
+	
+	n_Y = READuint16( &data[i]);
 	i += 2;
-#ifndef WORDS_BIGENDIAN
-	n_Y = byteswap16(n_Y);
-#endif
 
 	data_Y = &data[i];
 	i += n_Y;
@@ -217,6 +215,14 @@ int proc_dhe_dss_server_kx( GNUTLS_KEY key, opaque* data, int data_size) {
 int proc_dhe_dss_client_kx( GNUTLS_KEY key, opaque* data, int data_size) {
 	uint16 n_Y;
 	size_t _n_Y;
+	MPI g, p;
+	
+	cred = _gnutls_get_kx_cred( key, GNUTLS_KX_DHE_DSS, NULL);
+	if (cred==NULL) {
+		bits = DEFAULT_BITS; /* default */
+	} else {
+		bits = cred->bits;
+	}
 
 #if 0 /* removed. I do not know why - maybe I didn't get the protocol,
        * but openssl does not use that byte
@@ -227,18 +233,20 @@ int proc_dhe_dss_client_kx( GNUTLS_KEY key, opaque* data, int data_size) {
 	}
 #endif
 
-	memmove(&n_Y, &data[0], 2);
-#ifndef WORDS_BIGENDIAN
-	n_Y = byteswap16(n_Y);
-#endif
+	n_Y = READuint16( &data[0]);
 	_n_Y = n_Y;
+
 	if (gcry_mpi_scan(&key->client_Y,
 		      GCRYMPI_FMT_USG, &data[2], &_n_Y)) {
 		gnutls_assert();
 		return GNUTLS_E_MPI_SCAN_FAILED;
 	}
 
-	key->KEY = gnutls_calc_dh_key( key->client_Y, key->dh_secret);
+	g = gnutls_get_dh_params(&p, bits);
+	key->KEY = gnutls_calc_dh_key( key->client_Y, key->dh_secret, p);
+	
+	mpi_release(g);
+	mpi_release(p);
 
 	gnutls_mpi_release(key->client_Y);
 	gnutls_mpi_release(key->dh_secret);
@@ -267,3 +275,4 @@ int proc_dhe_dss_server_cert_vrfy( GNUTLS_KEY key, opaque* data, int data_size)
 	return 0;
 }
 
+#endif
\ No newline at end of file
