@@ -41,7 +41,7 @@ char *crypt_srpsha1(const char *username, const char *passwd,
 	int vsize, hash_len = gnutls_hash_get_algo_len(GNUTLS_MAC_SHA);
 	opaque *tmp;
 	uint8 *rtext, *csalt;
-	int rsalt_size;
+	int rsalt_size, len;
 
 	passwd_len = strlen(passwd);	/* we do not want the null */
 
@@ -61,8 +61,14 @@ char *crypt_srpsha1(const char *username, const char *passwd,
 		return NULL;
 	}
 	sp++;
-
-	rsalt_size = _gnutls_sbase64_decode(sp, strlen(sp), &csalt);
+	
+	len = (int)rindex(sp, ':') - (int)sp;
+	
+	rsalt_size = _gnutls_sbase64_decode(sp, len, &csalt);
+	if (rsalt_size < 0) {
+		gnutls_assert();
+		return NULL;
+	}
 
 	h1 = gnutls_hash_init(GNUTLS_MAC_SHA);
 	gnutls_hash(h1, csalt, rsalt_size);
@@ -81,7 +87,11 @@ char *crypt_srpsha1(const char *username, const char *passwd,
 		return NULL;
 	}
 
-	_gnutls_sbase64_encode(v, vsize, &rtext);
+	if (_gnutls_sbase64_encode(v, vsize, &rtext) < 0) {
+		gnutls_free(v);
+		gnutls_assert();
+		return NULL;
+	}
 	gnutls_free(v);
 
 	tmp =
@@ -113,6 +123,7 @@ char *crypt_srpsha1_wrapper(const char *username, const char *pass_new,
 
 	result_size = _gnutls_sbase64_encode(rand, salt, &result);
 	if (result_size < 0) {
+		_gnutls_free_rand(rand);
 		gnutls_assert();
 		return NULL;
 	}
