@@ -33,6 +33,7 @@
 #include "gnutls_algorithms.h"
 #include "gnutls_db.h"
 #include "gnutls_auth_int.h"
+#include "gnutls_num.h"
 #ifdef HAVE_ERRNO_H
 #include <errno.h>
 #endif
@@ -41,7 +42,6 @@
 # define EAGAIN EWOULDBLOCK
 #endif
 
-
 GNUTLS_Version gnutls_get_current_version(GNUTLS_STATE state) {
 GNUTLS_Version ver;
 	ver = state->connection_state.version;
@@ -56,164 +56,135 @@ int gnutls_is_secure_memory(const void* mem) {
 	return 0;
 }
 
+/**
+  * gnutls_set_lowat - Used to set the lowat value in order for select to check for pending data.
+  * @state: is a &GNUTLS_STATE structure.
+  * @num: is the low water value.
+  *
+  * Used to set the lowat value in order for select to check
+  * if there are pending data to socket buffer. Used only   
+  * if you have changed the default low water value (default is 1).
+  * Normally you will not need that function.
+  **/
 int gnutls_set_lowat(GNUTLS_STATE state, int num) {
 	state->gnutls_internals.lowat = num;
 	return 0;
 }
 
-/* This function initializes the state to null (null encryption etc...) */
+/**
+  * gnutls_init - This function initializes the state to null (null encryption etc...).
+  * @con_end: is used to indicate if this state is to be used for server or 
+  * client. Can be one of GNUTLS_CLIENT and GNUTLS_SERVER. 
+  * @state: is a pointer to a &GNUTLS_STATE structure.
+  *
+  * This function initializes the current state to null. Every state
+  * must be initialized before use, so internal structures can be allocated.
+  * This function allocates structures which can only be free'd
+  * by calling gnutls_deinit(). Returns zero on success.
+  **/
 int gnutls_init(GNUTLS_STATE * state, ConnectionEnd con_end)
 {
 	/* for gcrypt in order to be able to allocate memory */
-	gcry_set_allocation_handler(gnutls_malloc, secure_malloc,  gnutls_is_secure_memory, gnutls_realloc, free);
+	gcry_set_allocation_handler(gnutls_malloc, secure_malloc, gnutls_is_secure_memory, gnutls_realloc, free);
 
 	*state = gnutls_calloc(1, sizeof(GNUTLS_STATE_INT));
-	memset(*state, 0, sizeof(GNUTLS_STATE));
+	
 	(*state)->security_parameters.entity = con_end;
 
-/* Set the defaults (only to remind me that they should be allocated ) */
+/* Set the defaults for initial handshake */
 	(*state)->security_parameters.bulk_cipher_algorithm = GNUTLS_NULL_CIPHER;
 	(*state)->security_parameters.mac_algorithm = GNUTLS_NULL_MAC;
 	(*state)->security_parameters.compression_algorithm = GNUTLS_NULL_COMPRESSION;
 
-	(*state)->connection_state.read_compression_state = NULL;
-	(*state)->connection_state.read_mac_secret = NULL;
-	(*state)->connection_state.write_compression_state = NULL;
-	(*state)->connection_state.write_mac_secret = NULL;
-
-	(*state)->cipher_specs.server_write_mac_secret = NULL;
-	(*state)->cipher_specs.client_write_mac_secret = NULL;
-	(*state)->cipher_specs.server_write_IV = NULL;
-	(*state)->cipher_specs.client_write_IV = NULL;
-	(*state)->cipher_specs.server_write_key = NULL;
-	(*state)->cipher_specs.client_write_key = NULL;
-
-	(*state)->gnutls_internals.v2_hello = 0;
-	
-	(*state)->gnutls_internals.buffer = NULL;
-	/* SSL3 stuff */
-	(*state)->gnutls_internals.hash_buffer = NULL;
-	
-	(*state)->gnutls_internals.buffer_handshake = NULL;
 	(*state)->gnutls_internals.resumable = RESUME_TRUE;
 
 	gnutls_set_current_version ( (*state), GNUTLS_TLS1); /* default */
 
-	(*state)->gnutls_key = gnutls_malloc(sizeof(GNUTLS_KEY_A));
+	(*state)->gnutls_key = gnutls_calloc(1, sizeof(GNUTLS_KEY_A));
 
-	(*state)->gnutls_key->auth_info = NULL; /* no default auth_info */
-	(*state)->gnutls_key->auth_info_size = 0; /* no default auth_info */
-	(*state)->gnutls_key->cred = NULL; /* no credentials by default */
-	
-	(*state)->gnutls_key->KEY = NULL;
-	(*state)->gnutls_key->client_Y = NULL;
-	(*state)->gnutls_key->client_p = NULL;
-	(*state)->gnutls_key->client_g = NULL;
-	(*state)->gnutls_key->dh_secret = NULL;
-
-	(*state)->gnutls_key->A = NULL;
-	(*state)->gnutls_key->a = NULL;	
-	(*state)->gnutls_key->x = NULL;
-	(*state)->gnutls_key->u = NULL;
-	(*state)->gnutls_key->B = NULL;
-	(*state)->gnutls_key->b = NULL;	
-
-	(*state)->gnutls_internals.certificate_requested = 0;
-	(*state)->gnutls_internals.certificate_verify_needed = 0;
-
-	(*state)->gnutls_internals.MACAlgorithmPriority.algorithm_priority=NULL;
-	(*state)->gnutls_internals.MACAlgorithmPriority.algorithms=0;
-
-	(*state)->gnutls_internals.KXAlgorithmPriority.algorithm_priority=NULL;
-	(*state)->gnutls_internals.KXAlgorithmPriority.algorithms=0;
-
-	(*state)->gnutls_internals.BulkCipherAlgorithmPriority.algorithm_priority=NULL;
-	(*state)->gnutls_internals.BulkCipherAlgorithmPriority.algorithms=0;
-
-	(*state)->gnutls_internals.CompressionMethodPriority.algorithm_priority=NULL;
-	(*state)->gnutls_internals.CompressionMethodPriority.algorithms=0;
-
-	/* Set default priorities */
-	gnutls_set_cipher_priority( (*state), 2, GNUTLS_RIJNDAEL, GNUTLS_3DES);
-	gnutls_set_compression_priority( (*state), 1, GNUTLS_NULL_COMPRESSION);
-	gnutls_set_kx_priority( (*state), 1, GNUTLS_KX_ANON_DH);
-	gnutls_set_mac_priority( (*state), 2, GNUTLS_MAC_SHA, GNUTLS_MAC_MD5);
-
-	(*state)->security_parameters.session_id_size = 0;
-	(*state)->gnutls_internals.resumed_security_parameters.session_id_size = 0;
 	(*state)->gnutls_internals.resumed = RESUME_FALSE;
-	
-	(*state)->gnutls_internals.expire_time = 3600; /* one hour default */
 
-	(*state)->security_parameters.timestamp = 0;
+	(*state)->gnutls_internals.expire_time = DEFAULT_EXPIRE_TIME; /* one hour default */
 
-	/* gdbm db */
-	(*state)->gnutls_internals.db_name = NULL;
+	gnutls_set_lowat((*state), DEFAULT_LOWAT); /* the default for tcp */
 
-	gnutls_set_lowat((*state), 1); /* the default for tcp */
+	/* everything else not initialized here is initialized
+	 * as NULL or 0. This is why calloc is used.
+	 */
 	
 	return 0;
 }
 
 #define GNUTLS_FREE(x) if(x!=NULL) gnutls_free(x)
-/* This function clears all buffers associated with the state. */
-int gnutls_deinit(GNUTLS_STATE * state)
+/**
+  * gnutls_init - This function clears all buffers associated with the &state
+  * @state: is a &GNUTLS_STATE structure.
+  *
+  * This function clears all buffers associated with the &state.
+  **/
+int gnutls_deinit(GNUTLS_STATE state)
 {
 	/* if the session has failed abnormally it has to be removed from the db */
-	if ( (*state)->gnutls_internals.resumable==RESUME_FALSE) {
-		_gnutls_db_remove_session( (*state), (*state)->security_parameters.session_id, (*state)->security_parameters.session_id_size);
+	if ( state->gnutls_internals.resumable==RESUME_FALSE) {
+		_gnutls_db_remove_session( state, state->security_parameters.session_id, state->security_parameters.session_id_size);
 	}
 
 	/* remove auth info firstly */
-	GNUTLS_FREE((*state)->gnutls_key->auth_info);
-	
-	GNUTLS_FREE((*state)->connection_state.read_compression_state);
-	GNUTLS_FREE((*state)->connection_state.read_mac_secret);
-	GNUTLS_FREE((*state)->connection_state.write_compression_state);
-	GNUTLS_FREE((*state)->connection_state.write_mac_secret);
+	GNUTLS_FREE(state->gnutls_key->auth_info);
 
-	GNUTLS_FREE((*state)->gnutls_internals.buffer);
-	GNUTLS_FREE((*state)->gnutls_internals.buffer_handshake);
+#ifdef HAVE_LIBGDBM
+	/* close the database - resuming sessions */
+	if ( state->gnutls_internals.db_reader != NULL)
+		gdbm_close(state->gnutls_internals.db_reader);
+#endif
 
-	gnutls_clear_creds( *state);
+	GNUTLS_FREE(state->connection_state.read_compression_state);
+	GNUTLS_FREE(state->connection_state.read_mac_secret);
+	GNUTLS_FREE(state->connection_state.write_compression_state);
+	GNUTLS_FREE(state->connection_state.write_mac_secret);
 
-	if ((*state)->connection_state.read_cipher_state != NULL)
-		gnutls_cipher_deinit((*state)->connection_state.read_cipher_state);
-	if ((*state)->connection_state.write_cipher_state != NULL)
-		gnutls_cipher_deinit((*state)->connection_state.write_cipher_state);
+	GNUTLS_FREE(state->gnutls_internals.buffer);
+	GNUTLS_FREE(state->gnutls_internals.buffer_handshake);
 
-	secure_free((*state)->cipher_specs.server_write_mac_secret);
-	secure_free((*state)->cipher_specs.client_write_mac_secret);
-	secure_free((*state)->cipher_specs.server_write_IV);
-	secure_free((*state)->cipher_specs.client_write_IV);
-	secure_free((*state)->cipher_specs.server_write_key);
-	secure_free((*state)->cipher_specs.client_write_key);
+	gnutls_clear_creds( state);
 
-	mpi_release((*state)->gnutls_key->KEY);
-	mpi_release((*state)->gnutls_key->client_Y);
-	mpi_release((*state)->gnutls_key->client_p);
-	mpi_release((*state)->gnutls_key->client_g);
+	if (state->connection_state.read_cipher_state != NULL)
+		gnutls_cipher_deinit(state->connection_state.read_cipher_state);
+	if (state->connection_state.write_cipher_state != NULL)
+		gnutls_cipher_deinit(state->connection_state.write_cipher_state);
 
-	mpi_release((*state)->gnutls_key->u);
-	mpi_release((*state)->gnutls_key->a);
-	mpi_release((*state)->gnutls_key->x);
-	mpi_release((*state)->gnutls_key->A);
-	mpi_release((*state)->gnutls_key->B);
-	mpi_release((*state)->gnutls_key->b);
+	secure_free(state->cipher_specs.server_write_mac_secret);
+	secure_free(state->cipher_specs.client_write_mac_secret);
+	secure_free(state->cipher_specs.server_write_IV);
+	secure_free(state->cipher_specs.client_write_IV);
+	secure_free(state->cipher_specs.server_write_key);
+	secure_free(state->cipher_specs.client_write_key);
 
-	mpi_release((*state)->gnutls_key->dh_secret);
-	GNUTLS_FREE((*state)->gnutls_key);
+	mpi_release(state->gnutls_key->KEY);
+	mpi_release(state->gnutls_key->client_Y);
+	mpi_release(state->gnutls_key->client_p);
+	mpi_release(state->gnutls_key->client_g);
+
+	mpi_release(state->gnutls_key->u);
+	mpi_release(state->gnutls_key->a);
+	mpi_release(state->gnutls_key->x);
+	mpi_release(state->gnutls_key->A);
+	mpi_release(state->gnutls_key->B);
+	mpi_release(state->gnutls_key->b);
+
+	mpi_release(state->gnutls_key->dh_secret);
+	GNUTLS_FREE(state->gnutls_key);
 
 
 	/* free priorities */
-	GNUTLS_FREE((*state)->gnutls_internals.MACAlgorithmPriority.algorithm_priority);
-	GNUTLS_FREE((*state)->gnutls_internals.KXAlgorithmPriority.algorithm_priority);
-	GNUTLS_FREE((*state)->gnutls_internals.BulkCipherAlgorithmPriority.algorithm_priority);
-	GNUTLS_FREE((*state)->gnutls_internals.CompressionMethodPriority.algorithm_priority);
+	GNUTLS_FREE(state->gnutls_internals.MACAlgorithmPriority.algorithm_priority);
+	GNUTLS_FREE(state->gnutls_internals.KXAlgorithmPriority.algorithm_priority);
+	GNUTLS_FREE(state->gnutls_internals.BulkCipherAlgorithmPriority.algorithm_priority);
+	GNUTLS_FREE(state->gnutls_internals.CompressionMethodPriority.algorithm_priority);
 
-	GNUTLS_FREE((*state)->gnutls_internals.db_name);
+	GNUTLS_FREE(state->gnutls_internals.db_name);
 
-	GNUTLS_FREE(*state);
+	GNUTLS_FREE(state);
 	return 0;
 }
 
@@ -427,9 +398,16 @@ int _gnutls_send_alert(int cd, GNUTLS_STATE state, AlertLevel level, AlertDescri
 	memmove(&data[1], &desc, 1);
 
 	return gnutls_send_int(cd, state, GNUTLS_ALERT, data, 2, 0);
-
 }
 
+/**
+  * gnutls_close - This function terminates the current TLS/SSL connection.
+  * @cd: is a connection descriptor.
+  * @state: is a &GNUTLS_STATE structure.
+  *
+  * Terminates the current TLS/SSL connection. If the return value is 0
+  * you may continue using the TCP connection.
+  **/
 int gnutls_close(int cd, GNUTLS_STATE state)
 {
 	int ret;
@@ -460,16 +438,15 @@ int gnutls_close_nowait(int cd, GNUTLS_STATE state)
  * send (if called by the user the Content is specific)
  * It is intended to transfer data, under the current state.    
  */
-ssize_t gnutls_send_int(int cd, GNUTLS_STATE state, ContentType type, void *_data, size_t sizeofdata, int flags)
+ssize_t gnutls_send_int(int cd, GNUTLS_STATE state, ContentType type, const void *_data, size_t sizeofdata, int flags)
 {
 	uint8 *cipher;
 	int i, cipher_size;
 	int ret = 0;
 	int iterations;
-	uint16 length;
 	int Size;
 	uint8 headers[5];
-	uint8 *data=_data;
+	const uint8 *data=_data;
 
 	if (sizeofdata == 0)
 		return 0;
@@ -488,17 +465,18 @@ ssize_t gnutls_send_int(int cd, GNUTLS_STATE state, ContentType type, void *_dat
 	headers[0]=type;
 	headers[1]=_gnutls_version_get_major(state->connection_state.version);
 	headers[2]=_gnutls_version_get_minor(state->connection_state.version);
-	
+
+#ifdef HARD_DEBUG
+	fprintf(stderr, "Record: Sending Packet[%d] %s(%d) with length: %d\n",
+		(int) uint64touint32(&state->connection_state.read_sequence_number), _gnutls_packet2str(type), type, sizeofdata);
+#endif
+
 	for (i = 0; i < iterations; i++) {
 		cipher_size = _gnutls_encrypt( state, &data[i*Size], Size, &cipher, type);
 		if (cipher_size <= 0) return cipher_size; /* error */
+
+		WRITEuint16( cipher_size, &headers[3]);
 		
-#ifdef WORDS_BIGENDIAN
-		length = cipher_size;
-#else
-		length = byteswap16(cipher_size);
-#endif
-		memmove( &headers[3], &length, sizeof(uint16));
 		/* cipher does not have headers 
 		 * and DOES have size for them
 		 */
@@ -511,19 +489,21 @@ ssize_t gnutls_send_int(int cd, GNUTLS_STATE state, ContentType type, void *_dat
 			gnutls_assert();
 			return GNUTLS_E_UNABLE_SEND_DATA;
 		}
-		state->connection_state.write_sequence_number++;
+#ifdef HARD_DEBUG
+		fprintf(stderr, "Record: Sended Packet[%d] %s(%d) with length: %d\n",
+		(int) uint64touint32(&state->connection_state.read_sequence_number), _gnutls_packet2str(type), type, cipher_size);
+#endif
+
+		uint64pp( &state->connection_state.write_sequence_number);
 	}
 		/* rest data */
 	if (iterations > 1) {
 		Size = sizeofdata % MAX_ENC_LEN;
 		cipher_size = _gnutls_encrypt( state, &data[i*Size], Size, &cipher, type);
 		if (cipher_size<=0) return cipher_size;
-#ifdef WORDS_BIGENDIAN
-		length = cipher_size;
-#else
-		length = byteswap16(cipher_size);
-#endif
-		memmove( &headers[3], &length, sizeof(uint16));
+
+		WRITEuint16( cipher_size, &headers[3]);
+
 		memmove( cipher, headers, HEADER_SIZE);
 
 		cipher_size += HEADER_SIZE;
@@ -533,7 +513,8 @@ ssize_t gnutls_send_int(int cd, GNUTLS_STATE state, ContentType type, void *_dat
 			gnutls_assert();
 			return GNUTLS_E_UNABLE_SEND_DATA;
 		}
-		state->connection_state.write_sequence_number++;
+
+		uint64pp( &state->connection_state.write_sequence_number);
 	}
 
 	ret += sizeofdata;
@@ -548,7 +529,6 @@ ssize_t gnutls_send_int(int cd, GNUTLS_STATE state, ContentType type, void *_dat
  */
 ssize_t _gnutls_send_change_cipher_spec(int cd, GNUTLS_STATE state)
 {
-	uint16 length;
 	int ret = 0;
 	uint8 type=GNUTLS_CHANGE_CIPHER_SPEC;
 	char data[1] = { GNUTLS_TYPE_CHANGE_CIPHER_SPEC };
@@ -566,12 +546,7 @@ ssize_t _gnutls_send_change_cipher_spec(int cd, GNUTLS_STATE state)
 	fprintf(stderr, "ChangeCipherSpec was sent\n");
 #endif
 
-#ifdef WORDS_BIGENDIAN
-	length = (uint16)1;
-#else
-	length = byteswap16((uint16)1);
-#endif
-	memmove( &headers[3], &length, sizeof(uint16));
+	WRITEuint16( 1, &headers[3]);
 	
 	if (_gnutls_Write(cd, headers, 5) != 5) {
 		state->gnutls_internals.valid_connection = VALID_FALSE;
@@ -616,7 +591,7 @@ ssize_t gnutls_recv_int(int cd, GNUTLS_STATE state, ContentType type, char *data
 	int tmplen;
 	GNUTLS_Version version;
 	uint8 headers[HEADER_SIZE];
-	uint8 recv_type;
+	ContentType recv_type;
 	uint16 length;
 	uint8 *ciphertext;
 	int ret = 0;
@@ -672,17 +647,13 @@ ssize_t gnutls_recv_int(int cd, GNUTLS_STATE state, ContentType type, char *data
 	} else {
 		/* version 3.x 
 		 */
-		memcpy( &recv_type, &headers[0], 1);
+		recv_type = headers[0];
 		version = _gnutls_version_get( headers[1], headers[2]);
 
-		memcpy( &length, &headers[3], 2);
-#ifndef WORDS_BIGENDIAN
-		length = byteswap16(length);
-#endif
+		length = READuint16( &headers[3]);
 	}
 	
-
-	if (type != GNUTLS_HANDSHAKE && gnutls_get_current_version(state) != version) {
+	if ( gnutls_get_current_version(state) != version) {
 #ifdef DEBUG
 		fprintf(stderr, "Record: INVALID VERSION PACKET: (%d) %d.%d\n", headers[0], headers[1], headers[2]);
 #endif
@@ -695,9 +666,9 @@ ssize_t gnutls_recv_int(int cd, GNUTLS_STATE state, ContentType type, char *data
 
 #ifdef HARD_DEBUG
 	fprintf(stderr, "Record: Expected Packet[%d] %s(%d) with length: %d\n",
-		(int) state->connection_state.read_sequence_number, _gnutls_packet2str(type), type, sizeofdata);
+		(int) uint64touint32(&state->connection_state.read_sequence_number), _gnutls_packet2str(type), type, sizeofdata);
 	fprintf(stderr, "Record: Received Packet[%d] %s(%d) with length: %d\n",
-		(int) state->connection_state.read_sequence_number, _gnutls_packet2str(recv_type), recv_type, length);
+		(int) uint64touint32(&state->connection_state.read_sequence_number), _gnutls_packet2str(recv_type), recv_type, length);
 #endif
 
 	if (length > MAX_RECV_SIZE) {
@@ -725,6 +696,7 @@ ssize_t gnutls_recv_int(int cd, GNUTLS_STATE state, ContentType type, char *data
 		gnutls_assert();
 		return GNUTLS_E_UNEXPECTED_PACKET_LENGTH;		
 	}
+	
 /* ok now we are sure that we can read all the data - so
  * move on !
  */
@@ -735,8 +707,12 @@ ssize_t gnutls_recv_int(int cd, GNUTLS_STATE state, ContentType type, char *data
 		gnutls_assert();
 		return GNUTLS_E_UNKNOWN_ERROR;
 	}
-/* Read the whole packet - again? */	
-	if ( type==GNUTLS_APPLICATION_DATA) {
+
+/* Read the whole packet - again? 
+ * Here we keep RCVLOWAT bytes in the TCP buffers, only for
+ * APPLICATION_DATA data.
+ */	
+	if ( type==GNUTLS_APPLICATION_DATA && type==recv_type) {
 		/* get the data - but do not free the buffer in the kernel */
 		ret = _gnutls_Read(cd, ciphertext, length-RCVLOWAT, 0);
 		if (ret>=0)
@@ -792,6 +768,11 @@ ssize_t gnutls_recv_int(int cd, GNUTLS_STATE state, ContentType type, char *data
 		return tmplen;
 	}
 
+#ifdef HARD_DEBUG
+	fprintf(stderr, "Record: Decrypted Packet[%d] %s(%d) with length: %d\n",
+		(int) uint64touint32(&state->connection_state.read_sequence_number), _gnutls_packet2str(recv_type), recv_type, tmplen);
+#endif
+
 	gnutls_free(ciphertext);
 
 	if ( (recv_type == type) && (type == GNUTLS_APPLICATION_DATA || type == GNUTLS_HANDSHAKE)) {
@@ -804,6 +785,14 @@ ssize_t gnutls_recv_int(int cd, GNUTLS_STATE state, ContentType type, char *data
 #endif
 			state->gnutls_internals.last_alert = tmpdata[1];
 
+			/* Increase sequence number 
+			 * this is needed only here because we return immediately
+			 */
+			uint64pp( &state->connection_state.read_sequence_number);
+
+			/* if close notify is received and
+			 * the alert is not fatal
+			 */
 			if (tmpdata[1] == GNUTLS_CLOSE_NOTIFY && tmpdata[0] != GNUTLS_FATAL) {
 
 				/* If we have been expecting for an alert do 
@@ -811,26 +800,46 @@ ssize_t gnutls_recv_int(int cd, GNUTLS_STATE state, ContentType type, char *data
 				 */
 				if (type != GNUTLS_ALERT)
 					gnutls_close_nowait(cd, state);
-
+				
+				gnutls_free(tmpdata);
+				
 				return GNUTLS_E_CLOSURE_ALERT_RECEIVED;
 			} else {
+			
+				/* if the alert is FATAL or WARNING
+				 * return the apropriate message
+				 */
+			
+				ret = GNUTLS_E_WARNING_ALERT_RECEIVED;
 				if (tmpdata[0] == GNUTLS_FATAL) {
 					state->gnutls_internals.valid_connection = VALID_FALSE;
 					state->gnutls_internals.resumable = RESUME_FALSE;
 					
-					return GNUTLS_E_FATAL_ALERT_RECEIVED;
+					ret = GNUTLS_E_FATAL_ALERT_RECEIVED;
 				}
-				return GNUTLS_E_WARNING_ALERT_RECEIVED;
+
+				gnutls_free(tmpdata);
+
+				return ret;
 			}
 			break;
 
 		case GNUTLS_CHANGE_CIPHER_SPEC:
 			/* this packet is now handled above */
 			gnutls_assert();
+	
+			gnutls_free(tmpdata);
+			
 			return GNUTLS_E_UNEXPECTED_PACKET;
 		case GNUTLS_APPLICATION_DATA:
 			/* even if data is unexpected put it into the buffer */
 			gnutls_insertDataBuffer(recv_type, state, (void *) tmpdata, tmplen);
+			/* no peeked data to clear since this packet was unexpected */
+
+			break;
+		case GNUTLS_HANDSHAKE:
+			/* This is only legal if HELLO_REQUEST is received */
+
 			break;
 		default:
 #ifdef DEBUG
@@ -842,9 +851,8 @@ ssize_t gnutls_recv_int(int cd, GNUTLS_STATE state, ContentType type, char *data
 	}
 
 
-
 	/* Increase sequence number */
-	state->connection_state.read_sequence_number++;
+	uint64pp( &state->connection_state.read_sequence_number);
 
 
 	/* Get Application data from buffer */
@@ -859,26 +867,212 @@ ssize_t gnutls_recv_int(int cd, GNUTLS_STATE state, ContentType type, char *data
 		}
 		gnutls_free(tmpdata);
 	} else {
-		if (recv_type != GNUTLS_APPLICATION_DATA) {
-			gnutls_assert();
-			return GNUTLS_E_RECEIVED_BAD_MESSAGE;
+		if (recv_type == GNUTLS_HANDSHAKE) {
+			/* we may get a hello request */
+			ret = _gnutls_recv_hello_request( cd, state, tmpdata, tmplen);
+			if (ret < 0) {
+				gnutls_assert();
+			} else /* inform the caller */
+				ret = GNUTLS_E_GOT_HELLO_REQUEST;
 		} else {
-			ret = 0; /* ok */
+			gnutls_assert();
+			ret = GNUTLS_E_UNEXPECTED_PACKET; 
+				/* we didn't get what we wanted to 
+				 */
+			if (recv_type == GNUTLS_APPLICATION_DATA)
+				ret = GNUTLS_E_GOT_APPLICATION_DATA;
 		}
+		gnutls_free(tmpdata);
 	}
 
 	return ret;
 }
 
+/**
+  * gnutls_get_current_cipher - Returns the currently used cipher.
+  * @state: is a &GNUTLS_STATE structure.
+  *
+  * Returns the currently used cipher.
+  **/
 BulkCipherAlgorithm gnutls_get_current_cipher( GNUTLS_STATE state) {
 	return state->security_parameters.bulk_cipher_algorithm;
 }
+
+/**
+  * gnutls_get_current_kx - Returns the key exchange algorithm.
+  * @state: is a &GNUTLS_STATE structure.
+  *
+  * Returns the key exchange algorithm used in the last handshake.
+  **/
 KXAlgorithm gnutls_get_current_kx( GNUTLS_STATE state) {
 	return state->security_parameters.kx_algorithm;
 }
+
+/**
+  * gnutls_get_current_mac_algorithm - Returns the currently used mac algorithm.
+  * @state: is a &GNUTLS_STATE structure.
+  *
+  * Returns the currently used mac algorithm.
+  **/
 MACAlgorithm gnutls_get_current_mac_algorithm( GNUTLS_STATE state) {
 	return state->security_parameters.mac_algorithm;
 }
+
+/**
+  * gnutls_get_current_compression_method - Returns the currently used compression algorithm.
+  * @state: is a &GNUTLS_STATE structure.
+  *
+  * Returns the currently used compression method.
+  **/
 CompressionMethod gnutls_get_current_compression_method( GNUTLS_STATE state) {
 	return state->security_parameters.compression_algorithm;
 }
+
+/* Taken from libgcrypt */
+
+static const char*
+parse_version_number( const char *s, int *number )
+{
+    int val = 0;
+
+    if( *s == '0' && isdigit(s[1]) )
+	return NULL; /* leading zeros are not allowed */
+    for ( ; isdigit(*s); s++ ) {
+	val *= 10;
+	val += *s - '0';
+    }
+    *number = val;
+    return val < 0? NULL : s;
+}
+
+
+static const char *
+parse_version_string( const char *s, int *major, int *minor, int *micro )
+{
+    s = parse_version_number( s, major );
+    if( !s || *s != '.' )
+	return NULL;
+    s++;
+    s = parse_version_number( s, minor );
+    if( !s || *s != '.' )
+	return NULL;
+    s++;
+    s = parse_version_number( s, micro );
+    if( !s )
+	return NULL;
+    return s; /* patchlevel */
+}
+
+/****************
+ * Check that the the version of the library is at minimum the requested one
+ * and return the version string; return NULL if the condition is not
+ * satisfied.  If a NULL is passed to this function, no check is done,
+ * but the version string is simply returned.
+ */
+const char *
+gnutls_check_version( const char *req_version )
+{
+    const char *ver = GNUTLS_VERSION;
+    int my_major, my_minor, my_micro;
+    int rq_major, rq_minor, rq_micro;
+    const char *my_plvl, *rq_plvl;
+
+    if ( !req_version )
+	return ver;
+
+    my_plvl = parse_version_string( ver, &my_major, &my_minor, &my_micro );
+    if ( !my_plvl )
+	return NULL;  /* very strange our own version is bogus */
+    rq_plvl = parse_version_string( req_version, &rq_major, &rq_minor,
+								&rq_micro );
+    if ( !rq_plvl )
+	return NULL;  /* req version string is invalid */
+
+    if ( my_major > rq_major
+	|| (my_major == rq_major && my_minor > rq_minor)
+	|| (my_major == rq_major && my_minor == rq_minor
+				 && my_micro > rq_micro)
+	|| (my_major == rq_major && my_minor == rq_minor
+				 && my_micro == rq_micro
+				 && strcmp( my_plvl, rq_plvl ) >= 0) ) {
+	return ver;
+    }
+    return NULL;
+}
+
+/**
+  * gnutls_get_last_alert - Returns the last alert number received.
+  * @state: is a &GNUTLS_STATE structure.
+  *
+  * Returns the last alert number received. This function
+  * should be called if %GNUTLS_E_WARNING_ALERT_RECEIVED or
+  * %GNUTLS_E_FATAL_ALERT_RECEIVED has been returned by a gnutls function.
+  * The peer may send alerts if he thinks some things were not 
+  * right. Check gnutls.h for the available alert descriptions.
+  **/
+AlertDescription gnutls_get_last_alert( GNUTLS_STATE state) {
+	return state->gnutls_internals.last_alert;
+}
+
+/**
+  * gnutls_send - sends to the peer the specified data
+  * @cd: is a connection descriptor
+  * @state: is a &GNUTLS_STATE structure.
+  * @data: contains the data to send
+  * @sizeofdata: is the length of the data
+  * @flags: contains the flags to pass to send() function.
+  *
+  * This function has the same semantics as send() has. The only
+  * difference is that is accepts a GNUTLS state. Currently flags cannot
+  * be anything except 0.
+  **/
+ssize_t gnutls_send(int cd, GNUTLS_STATE state, const void *data, size_t sizeofdata, int flags) {
+	return gnutls_send_int( cd, state, GNUTLS_APPLICATION_DATA, data, sizeofdata, flags);
+}
+
+/**
+  * gnutls_recv - receives data from the TLS connection
+  * @cd: is a connection descriptor
+  * @state: is a &GNUTLS_STATE structure.
+  * @data: contains the data to send
+  * @sizeofdata: is the length of the data
+  * @flags: contains the flags to pass to recv() function.
+  *
+  * This function has the same semantics as recv() has. The only
+  * difference is that is accepts a GNUTLS state. Flags are the flags
+  * passed to recv() and should be used with care in gnutls.  
+  * The only acceptable flag is currently MSG_DONTWAIT. In that case,
+  * if the socket is set to non blocking IO it will return GNUTLS_E_AGAIN,
+  * if there are no data in the socket. 
+  **/
+ssize_t gnutls_recv(int cd, GNUTLS_STATE state, void *data, size_t sizeofdata, int flags) {
+	return gnutls_recv_int( cd, state, GNUTLS_APPLICATION_DATA, data, sizeofdata, flags);
+}
+
+/**
+  * gnutls_write - sends to the peer the specified data
+  * @cd: is a connection descriptor
+  * @state: is a &GNUTLS_STATE structure.
+  * @data: contains the data to send
+  * @sizeofdata: is the length of the data
+  *
+  * This function has the same semantics as write() has. The only
+  * difference is that is accepts a GNUTLS state.
+  **/
+ssize_t gnutls_write(int cd, GNUTLS_STATE state, const void *data, size_t sizeofdata) {
+	return gnutls_send_int( cd, state, GNUTLS_APPLICATION_DATA, data, sizeofdata, 0);
+}
+
+/**
+  * gnutls_read - reads data from the TLS connection
+  * @cd: is a connection descriptor
+  * @state: is a &GNUTLS_STATE structure.
+  * @data: contains the data to send
+  * @sizeofdata: is the length of the data
+  *
+  * This function has the same semantics as read() has. The only
+  * difference is that is accepts a GNUTLS state. 
+  **/
+ssize_t gnutls_read(int cd, GNUTLS_STATE state, void *data, size_t sizeofdata) {
+	return gnutls_recv_int( cd, state, GNUTLS_APPLICATION_DATA, data, sizeofdata, 0);
+}
