@@ -25,7 +25,7 @@
 
 /* include all the kx handler's definitions */
 #include "auth_anon.h"
-#include "auth_dhe_dss.h"
+/* #include "auth_dhe_dss.h" */
 #include "auth_srp.h"
 
 #define MAX_CIPHER 256
@@ -38,20 +38,20 @@
 
 typedef struct {
 	char *name;
-	GNUTLS_Version id; /* gnutls internal version number */
-	int major; /* defined by the protocol */
-	int minor; /* defined by the protocol */
-	int supported;	/* 0 not supported, > 0 is supported */
+	GNUTLS_Version id;	/* gnutls internal version number */
+	int major;		/* defined by the protocol */
+	int minor;		/* defined by the protocol */
+	int supported;		/* 0 not supported, > 0 is supported */
 } gnutls_version_entry;
 
-static gnutls_version_entry sup_versions[] = {
+static const gnutls_version_entry sup_versions[] = {
 	{"SSL3", GNUTLS_SSL3, 3, 0, 1},
 	{"TLS1", GNUTLS_TLS1, 3, 1, 1},
 	{0}
 };
 
 #define GNUTLS_VERSION_LOOP(b) \
-        gnutls_version_entry *p; \
+        const gnutls_version_entry *p; \
                 for(p = sup_versions; p->name != NULL; p++) { b ; }
 
 #define GNUTLS_VERSION_ALG_LOOP(a) \
@@ -71,7 +71,7 @@ struct gnutls_cipher_entry {
 };
 typedef struct gnutls_cipher_entry gnutls_cipher_entry;
 
-static gnutls_cipher_entry algorithms[] = {
+static const gnutls_cipher_entry algorithms[] = {
 	GNUTLS_CIPHER_ENTRY(GNUTLS_3DES, 8, 24, 1, 8),
 	GNUTLS_CIPHER_ENTRY(GNUTLS_RIJNDAEL, 16, 16, 1, 16),
 	GNUTLS_CIPHER_ENTRY(GNUTLS_RIJNDAEL256, 16, 32, 1, 16),
@@ -82,7 +82,7 @@ static gnutls_cipher_entry algorithms[] = {
 };
 
 #define GNUTLS_LOOP(b) \
-        gnutls_cipher_entry *p; \
+        const gnutls_cipher_entry *p; \
                 for(p = algorithms; p->name != NULL; p++) { b ; }
 
 #define GNUTLS_ALG_LOOP(a) \
@@ -99,7 +99,7 @@ struct gnutls_hash_entry {
 };
 typedef struct gnutls_hash_entry gnutls_hash_entry;
 
-static gnutls_hash_entry hash_algorithms[] = {
+static const gnutls_hash_entry hash_algorithms[] = {
 	GNUTLS_HASH_ENTRY(GNUTLS_MAC_SHA, 20),
 	GNUTLS_HASH_ENTRY(GNUTLS_MAC_MD5, 16),
 	GNUTLS_HASH_ENTRY(GNUTLS_NULL_MAC, 0),
@@ -107,7 +107,7 @@ static gnutls_hash_entry hash_algorithms[] = {
 };
 
 #define GNUTLS_HASH_LOOP(b) \
-        gnutls_hash_entry *p; \
+        const gnutls_hash_entry *p; \
                 for(p = hash_algorithms; p->name != NULL; p++) { b ; }
 
 #define GNUTLS_HASH_ALG_LOOP(a) \
@@ -115,28 +115,31 @@ static gnutls_hash_entry hash_algorithms[] = {
 
 
 /* Compression Section */
-#define GNUTLS_COMPRESSION_ENTRY(name) \
-	{ #name, name }
+#define GNUTLS_COMPRESSION_ENTRY(name, id) \
+	{ #name, name, id }
 
 struct gnutls_compression_entry {
 	char *name;
 	CompressionMethod id;
+	int num; /* the number reserved in TLS for the specific compression method */
 };
 
 typedef struct gnutls_compression_entry gnutls_compression_entry;
-static gnutls_compression_entry compression_algorithms[] = {
-	GNUTLS_COMPRESSION_ENTRY(GNUTLS_NULL_COMPRESSION),
+static const gnutls_compression_entry compression_algorithms[] = {
+	GNUTLS_COMPRESSION_ENTRY(GNUTLS_NULL_COMPRESSION, 0),
 #ifdef HAVE_LIBZ
-	GNUTLS_COMPRESSION_ENTRY(GNUTLS_ZLIB),
+	GNUTLS_COMPRESSION_ENTRY(GNUTLS_ZLIB, 224),
 #endif
 	{0}
 };
 
 #define GNUTLS_COMPRESSION_LOOP(b) \
-        gnutls_compression_entry *p; \
+        const gnutls_compression_entry *p; \
                 for(p = compression_algorithms; p->name != NULL; p++) { b ; }
 #define GNUTLS_COMPRESSION_ALG_LOOP(a) \
                         GNUTLS_COMPRESSION_LOOP( if(p->id == algorithm) { a; break; } )
+#define GNUTLS_COMPRESSION_ALG_LOOP_NUM(a) \
+                        GNUTLS_COMPRESSION_LOOP( if(p->num == num) { a; break; } )
 
 
 /* Key Exchange Section */
@@ -154,19 +157,21 @@ struct gnutls_kx_algo_entry {
 };
 typedef struct gnutls_kx_algo_entry gnutls_kx_algo_entry;
 
-static gnutls_kx_algo_entry kx_algorithms[] = {
-	GNUTLS_KX_ALGO_ENTRY(GNUTLS_KX_ANON_DH, 0, 0, 0, 1, &anon_auth_struct),
-	GNUTLS_KX_ALGO_ENTRY(GNUTLS_KX_RSA,     1, 1, 1, 0, NULL),
-	GNUTLS_KX_ALGO_ENTRY(GNUTLS_KX_DHE_DSS, 1, 1, 0, 0, &dhe_dss_auth_struct),
+static const gnutls_kx_algo_entry kx_algorithms[] = {
+	GNUTLS_KX_ALGO_ENTRY(GNUTLS_KX_DH_ANON, 0, 0, 0, 1,
+			     &anon_auth_struct),
+	GNUTLS_KX_ALGO_ENTRY(GNUTLS_KX_RSA, 1, 1, 1, 0, NULL),
+/*	GNUTLS_KX_ALGO_ENTRY(GNUTLS_KX_DHE_DSS, 1, 1, 0, 0,
+			     &dhe_dss_auth_struct),*/
 	GNUTLS_KX_ALGO_ENTRY(GNUTLS_KX_DHE_RSA, 1, 1, 0, 0, NULL),
-	GNUTLS_KX_ALGO_ENTRY(GNUTLS_KX_DH_DSS,  1, 1, 0, 0, NULL),
-	GNUTLS_KX_ALGO_ENTRY(GNUTLS_KX_DH_RSA,  1, 1, 0, 0, NULL),
-	GNUTLS_KX_ALGO_ENTRY(GNUTLS_KX_SRP,     0, 0, 0, 0, &srp_auth_struct),
+	GNUTLS_KX_ALGO_ENTRY(GNUTLS_KX_DH_DSS, 1, 1, 0, 0, NULL),
+	GNUTLS_KX_ALGO_ENTRY(GNUTLS_KX_DH_RSA, 1, 1, 0, 0, NULL),
+	GNUTLS_KX_ALGO_ENTRY(GNUTLS_KX_SRP, 0, 0, 0, 0, &srp_auth_struct),
 	{0}
 };
 
 #define GNUTLS_KX_LOOP(b) \
-        gnutls_kx_algo_entry *p; \
+        const gnutls_kx_algo_entry *p; \
                 for(p = kx_algorithms; p->name != NULL; p++) { b ; }
 
 #define GNUTLS_KX_ALG_LOOP(a) \
@@ -190,56 +195,70 @@ typedef struct {
 #define GNUTLS_DH_anon_ARCFOUR_MD5 { 0x00, 0x18 }
 #define GNUTLS_DH_anon_RIJNDAEL_128_CBC_SHA { 0x00, 0x34 }
 #define GNUTLS_DH_anon_RIJNDAEL_256_CBC_SHA { 0x00, 0x3A }
-#define GNUTLS_DH_anon_TWOFISH_128_CBC_SHA { 0xFF, 0x50 }
+/* Twofish is a gnutls extension */
+#define GNUTLS_DH_anon_TWOFISH_128_CBC_SHA { 0xF6, 0x50 }
 
-#define GNUTLS_SRP_3DES_EDE_CBC_SHA { 0xF5, 0x1B }
-#define GNUTLS_SRP_ARCFOUR_MD5 { 0xF5, 0x18 }
-#define GNUTLS_SRP_RIJNDAEL_128_CBC_SHA { 0xF5, 0x34 }
-#define GNUTLS_SRP_RIJNDAEL_256_CBC_SHA { 0xF5, 0x3A }
-#define GNUTLS_SRP_TWOFISH_128_CBC_SHA { 0xF5, 0x50 }
+/* SRP is a gnutls extension - for now */
+#define GNUTLS_SRP_3DES_EDE_CBC_SHA { 0xF6, 0x60 }
+#define GNUTLS_SRP_ARCFOUR_MD5 { 0xF6, 0x61 }
+#define GNUTLS_SRP_RIJNDAEL_128_CBC_SHA { 0xF6, 0x62 }
+#define GNUTLS_SRP_RIJNDAEL_256_CBC_SHA { 0xF6, 0x63 }
+#define GNUTLS_SRP_TWOFISH_128_CBC_SHA { 0xF6, 0x64 }
 
-#define GNUTLS_DH_DSS_3DES_EDE_CBC_SHA { 0x00, 0x0D }
-#define GNUTLS_DH_RSA_3DES_EDE_CBC_SHA { 0x00, 0x10 }
-#define GNUTLS_DHE_DSS_3DES_EDE_CBC_SHA { 0x00, 0x13 }
-#define GNUTLS_DHE_RSA_3DES_EDE_CBC_SHA { 0x00, 0x16 }
+/* RSA */
 #define GNUTLS_RSA_ARCFOUR_SHA { 0x00, 0x05 }
 #define GNUTLS_RSA_ARCFOUR_MD5 { 0x00, 0x04 }
 #define GNUTLS_RSA_3DES_EDE_CBC_SHA { 0x00, 0x0A }
 #define GNUTLS_RSA_DES_CBC_SHA { 0x00, 0x09 }
-#define GNUTLS_DH_DSS_DES_CBC_SHA { 0x00, 0x0C }
-#define GNUTLS_DH_RSA_DES_CBC_SHA { 0x00, 0x0F }
-#define GNUTLS_DHE_DSS_DES_CBC_SHA { 0x00, 0x12 }
-#define GNUTLS_DHE_RSA_DES_CBC_SHA { 0x00, 0x15 }
-
 #define GNUTLS_RSA_RIJNDAEL_128_CBC_SHA { 0x00, 0x2F }
-#define GNUTLS_DH_DSS_RIJNDAEL_128_CBC_SHA { 0x00, 0x30 }
-#define GNUTLS_DH_RSA_RIJNDAEL_128_CBC_SHA { 0x00, 0x31 }
-#define GNUTLS_DHE_DSS_RIJNDAEL_128_CBC_SHA { 0x00, 0x32 }
-#define GNUTLS_DHE_RSA_RIJNDAEL_128_CBC_SHA { 0x00, 0x33 }
-
 #define GNUTLS_RSA_RIJNDAEL_256_CBC_SHA { 0x00, 0x35 }
+#define GNUTLS_RSA_TWOFISH_128_CBC_SHA { 0xF6, 0x51 }
+
+/* DH_DSS */
+#define GNUTLS_DH_DSS_RIJNDAEL_128_CBC_SHA { 0x00, 0x30 }
+#define GNUTLS_DH_DSS_TWOFISH_128_CBC_SHA { 0xF6, 0x52 }
+#define GNUTLS_DH_DSS_DES_CBC_SHA { 0x00, 0x0C }
 #define GNUTLS_DH_DSS_RIJNDAEL_256_CBC_SHA { 0x00, 0x36 }
-#define GNUTLS_DH_RSA_RIJNDAEL_256_CBC_SHA { 0x00, 0x37 }
+#define GNUTLS_DH_DSS_3DES_EDE_CBC_SHA { 0x00, 0x0D }
+
+/* DHE_DSS */
 #define GNUTLS_DHE_DSS_RIJNDAEL_256_CBC_SHA { 0x00, 0x38 }
+#define GNUTLS_DHE_DSS_RIJNDAEL_128_CBC_SHA { 0x00, 0x32 }
+#define GNUTLS_DHE_DSS_DES_CBC_SHA { 0x00, 0x12 }
+#define GNUTLS_DHE_DSS_TWOFISH_128_CBC_SHA { 0xF6, 0x54 }
+#define GNUTLS_DHE_DSS_3DES_EDE_CBC_SHA { 0x00, 0x13 }
+
+/* DHE_RSA */
+#define GNUTLS_DHE_RSA_TWOFISH_128_CBC_SHA { 0xF6, 0x55 }
+#define GNUTLS_DHE_RSA_3DES_EDE_CBC_SHA { 0x00, 0x16 }
+#define GNUTLS_DHE_RSA_DES_CBC_SHA { 0x00, 0x15 }
+#define GNUTLS_DHE_RSA_RIJNDAEL_128_CBC_SHA { 0x00, 0x33 }
 #define GNUTLS_DHE_RSA_RIJNDAEL_256_CBC_SHA { 0x00, 0x39 }
 
+/* DH_RSA */
+#define GNUTLS_DH_RSA_TWOFISH_128_CBC_SHA { 0xF6, 0x53 }
+#define GNUTLS_DH_RSA_DES_CBC_SHA { 0x00, 0x0F }
+#define GNUTLS_DH_RSA_3DES_EDE_CBC_SHA { 0x00, 0x10 }
+#define GNUTLS_DH_RSA_RIJNDAEL_256_CBC_SHA { 0x00, 0x37 }
+#define GNUTLS_DH_RSA_RIJNDAEL_128_CBC_SHA { 0x00, 0x31 }
 
-static gnutls_cipher_suite_entry cs_algorithms[] = {
-	/* anon */
+
+static const gnutls_cipher_suite_entry cs_algorithms[] = {
+	/* DH_anon */
 	GNUTLS_CIPHER_SUITE_ENTRY(GNUTLS_DH_anon_ARCFOUR_MD5,
 				  GNUTLS_ARCFOUR,
-				  GNUTLS_KX_ANON_DH, GNUTLS_MAC_MD5),
+				  GNUTLS_KX_DH_ANON, GNUTLS_MAC_MD5),
 	GNUTLS_CIPHER_SUITE_ENTRY(GNUTLS_DH_anon_3DES_EDE_CBC_SHA,
-				  GNUTLS_3DES, GNUTLS_KX_ANON_DH,
+				  GNUTLS_3DES, GNUTLS_KX_DH_ANON,
 				  GNUTLS_MAC_SHA),
 	GNUTLS_CIPHER_SUITE_ENTRY(GNUTLS_DH_anon_RIJNDAEL_128_CBC_SHA,
-				  GNUTLS_RIJNDAEL, GNUTLS_KX_ANON_DH,
+				  GNUTLS_RIJNDAEL, GNUTLS_KX_DH_ANON,
 				  GNUTLS_MAC_SHA),
 	GNUTLS_CIPHER_SUITE_ENTRY(GNUTLS_DH_anon_RIJNDAEL_256_CBC_SHA,
-				  GNUTLS_RIJNDAEL256, GNUTLS_KX_ANON_DH,
+				  GNUTLS_RIJNDAEL256, GNUTLS_KX_DH_ANON,
 				  GNUTLS_MAC_SHA),
 	GNUTLS_CIPHER_SUITE_ENTRY(GNUTLS_DH_anon_TWOFISH_128_CBC_SHA,
-				  GNUTLS_TWOFISH, GNUTLS_KX_ANON_DH,
+				  GNUTLS_TWOFISH, GNUTLS_KX_DH_ANON,
 				  GNUTLS_MAC_SHA),
 
 	/* SRP */
@@ -259,19 +278,63 @@ static gnutls_cipher_suite_entry cs_algorithms[] = {
 				  GNUTLS_TWOFISH, GNUTLS_KX_SRP,
 				  GNUTLS_MAC_SHA),
 
-	/* other! */
+	/* DH_DSS */
 	GNUTLS_CIPHER_SUITE_ENTRY(GNUTLS_DH_DSS_3DES_EDE_CBC_SHA,
 				  GNUTLS_3DES,
 				  GNUTLS_KX_DH_DSS, GNUTLS_MAC_SHA),
+	GNUTLS_CIPHER_SUITE_ENTRY(GNUTLS_DH_DSS_RIJNDAEL_128_CBC_SHA,
+				  GNUTLS_RIJNDAEL, GNUTLS_KX_DH_DSS,
+				  GNUTLS_MAC_SHA),
+	GNUTLS_CIPHER_SUITE_ENTRY(GNUTLS_DH_DSS_RIJNDAEL_256_CBC_SHA,
+				  GNUTLS_RIJNDAEL256, GNUTLS_KX_DH_DSS,
+				  GNUTLS_MAC_SHA),
+	GNUTLS_CIPHER_SUITE_ENTRY(GNUTLS_DH_DSS_TWOFISH_128_CBC_SHA,
+				  GNUTLS_TWOFISH, GNUTLS_KX_DH_DSS,
+				  GNUTLS_MAC_SHA),
+
+	/* DH_RSA */ 
 	GNUTLS_CIPHER_SUITE_ENTRY(GNUTLS_DH_RSA_3DES_EDE_CBC_SHA,
 				  GNUTLS_3DES,
 				  GNUTLS_KX_DH_RSA, GNUTLS_MAC_SHA),
+	GNUTLS_CIPHER_SUITE_ENTRY(GNUTLS_DH_RSA_RIJNDAEL_128_CBC_SHA,
+				  GNUTLS_RIJNDAEL, GNUTLS_KX_DH_RSA,
+				  GNUTLS_MAC_SHA),
+	GNUTLS_CIPHER_SUITE_ENTRY(GNUTLS_DH_RSA_RIJNDAEL_256_CBC_SHA,
+				  GNUTLS_RIJNDAEL256, GNUTLS_KX_DH_RSA,
+				  GNUTLS_MAC_SHA),
+	GNUTLS_CIPHER_SUITE_ENTRY(GNUTLS_DH_RSA_TWOFISH_128_CBC_SHA,
+				  GNUTLS_TWOFISH, GNUTLS_KX_DH_RSA,
+				  GNUTLS_MAC_SHA),
+
+	/* DHE_DSS */
+	GNUTLS_CIPHER_SUITE_ENTRY(GNUTLS_DHE_DSS_TWOFISH_128_CBC_SHA,
+				  GNUTLS_TWOFISH, GNUTLS_KX_DHE_DSS,
+				  GNUTLS_MAC_SHA),
 	GNUTLS_CIPHER_SUITE_ENTRY(GNUTLS_DHE_DSS_3DES_EDE_CBC_SHA,
 				  GNUTLS_3DES, GNUTLS_KX_DHE_DSS,
 				  GNUTLS_MAC_SHA),
+	GNUTLS_CIPHER_SUITE_ENTRY(GNUTLS_DHE_DSS_RIJNDAEL_128_CBC_SHA,
+				  GNUTLS_RIJNDAEL, GNUTLS_KX_DHE_DSS,
+				  GNUTLS_MAC_SHA),
+	GNUTLS_CIPHER_SUITE_ENTRY(GNUTLS_DHE_DSS_RIJNDAEL_256_CBC_SHA,
+				  GNUTLS_RIJNDAEL256, GNUTLS_KX_DHE_DSS,
+				  GNUTLS_MAC_SHA),
+
+	/* DHE_RSA */
+	GNUTLS_CIPHER_SUITE_ENTRY(GNUTLS_DHE_RSA_TWOFISH_128_CBC_SHA,
+				  GNUTLS_TWOFISH, GNUTLS_KX_DHE_RSA,
+				  GNUTLS_MAC_SHA),
 	GNUTLS_CIPHER_SUITE_ENTRY(GNUTLS_DHE_RSA_3DES_EDE_CBC_SHA,
 				  GNUTLS_3DES, GNUTLS_KX_DHE_RSA,
 				  GNUTLS_MAC_SHA),
+	GNUTLS_CIPHER_SUITE_ENTRY(GNUTLS_DHE_RSA_RIJNDAEL_128_CBC_SHA,
+				  GNUTLS_RIJNDAEL, GNUTLS_KX_DHE_RSA,
+				  GNUTLS_MAC_SHA),
+	GNUTLS_CIPHER_SUITE_ENTRY(GNUTLS_DHE_RSA_RIJNDAEL_256_CBC_SHA,
+				  GNUTLS_RIJNDAEL256, GNUTLS_KX_DHE_RSA,
+				  GNUTLS_MAC_SHA),
+
+	/* RSA */
 	GNUTLS_CIPHER_SUITE_ENTRY(GNUTLS_RSA_ARCFOUR_SHA,
 				  GNUTLS_ARCFOUR,
 				  GNUTLS_KX_RSA, GNUTLS_MAC_SHA),
@@ -284,38 +347,18 @@ static gnutls_cipher_suite_entry cs_algorithms[] = {
 	GNUTLS_CIPHER_SUITE_ENTRY(GNUTLS_RSA_RIJNDAEL_128_CBC_SHA,
 				  GNUTLS_RIJNDAEL, GNUTLS_KX_RSA,
 				  GNUTLS_MAC_SHA),
-	GNUTLS_CIPHER_SUITE_ENTRY(GNUTLS_DH_DSS_RIJNDAEL_128_CBC_SHA,
-				  GNUTLS_RIJNDAEL, GNUTLS_KX_DH_DSS,
-				  GNUTLS_MAC_SHA),
-	GNUTLS_CIPHER_SUITE_ENTRY(GNUTLS_DH_RSA_RIJNDAEL_128_CBC_SHA,
-				  GNUTLS_RIJNDAEL, GNUTLS_KX_DH_RSA,
-				  GNUTLS_MAC_SHA),
-	GNUTLS_CIPHER_SUITE_ENTRY(GNUTLS_DHE_DSS_RIJNDAEL_128_CBC_SHA,
-				  GNUTLS_RIJNDAEL, GNUTLS_KX_DHE_DSS,
-				  GNUTLS_MAC_SHA),
-	GNUTLS_CIPHER_SUITE_ENTRY(GNUTLS_DHE_RSA_RIJNDAEL_128_CBC_SHA,
-				  GNUTLS_RIJNDAEL, GNUTLS_KX_DHE_RSA,
-				  GNUTLS_MAC_SHA),
 	GNUTLS_CIPHER_SUITE_ENTRY(GNUTLS_RSA_RIJNDAEL_256_CBC_SHA,
 				  GNUTLS_RIJNDAEL256, GNUTLS_KX_RSA,
 				  GNUTLS_MAC_SHA),
-	GNUTLS_CIPHER_SUITE_ENTRY(GNUTLS_DH_DSS_RIJNDAEL_256_CBC_SHA,
-				  GNUTLS_RIJNDAEL256, GNUTLS_KX_DH_DSS,
-				  GNUTLS_MAC_SHA),
-	GNUTLS_CIPHER_SUITE_ENTRY(GNUTLS_DH_RSA_RIJNDAEL_256_CBC_SHA,
-				  GNUTLS_RIJNDAEL256, GNUTLS_KX_DH_RSA,
-				  GNUTLS_MAC_SHA),
-	GNUTLS_CIPHER_SUITE_ENTRY(GNUTLS_DHE_DSS_RIJNDAEL_256_CBC_SHA,
-				  GNUTLS_RIJNDAEL256, GNUTLS_KX_DHE_DSS,
-				  GNUTLS_MAC_SHA),
-	GNUTLS_CIPHER_SUITE_ENTRY(GNUTLS_DHE_RSA_RIJNDAEL_256_CBC_SHA,
-				  GNUTLS_RIJNDAEL256, GNUTLS_KX_DHE_RSA,
+	GNUTLS_CIPHER_SUITE_ENTRY(GNUTLS_RSA_TWOFISH_128_CBC_SHA,
+				  GNUTLS_TWOFISH, GNUTLS_KX_RSA,
 				  GNUTLS_MAC_SHA),
+
 	{0}
 };
 
 #define GNUTLS_CIPHER_SUITE_LOOP(b) \
-        gnutls_cipher_suite_entry *p; \
+        const gnutls_cipher_suite_entry *p; \
                 for(p = cs_algorithms; p->name != NULL; p++) { b ; }
 
 #define GNUTLS_CIPHER_SUITE_ALG_LOOP(a) \
@@ -352,12 +395,20 @@ inline int _gnutls_mac_priority(GNUTLS_STATE state, MACAlgorithm algorithm)
 	     i++) {
 		if (state->gnutls_internals.
 		    MACAlgorithmPriority.algorithm_priority[i] ==
-		    algorithm) return i;
+		    algorithm)
+			return i;
 	}
 	return -1;
 }
 
-char *_gnutls_mac_get_name(MACAlgorithm algorithm)
+/**
+  * gnutls_mac_get_name - Returns a string with the name of the specified mac algorithm
+  * @algorithm: is a MAC algorithm
+  *
+  * Returns an allocated (with malloc) string that contains the name 
+  * of the specified MAC algorithm.
+  **/
+char *gnutls_mac_get_name(MACAlgorithm algorithm)
 {
 	char *ret = NULL;
 	char *pointerTo_;
@@ -412,12 +463,20 @@ inline
 	     i++) {
 		if (state->gnutls_internals.
 		    CompressionMethodPriority.algorithm_priority[i] ==
-		    algorithm) return i;
+		    algorithm)
+			return i;
 	}
 	return -1;
 }
 
-char *_gnutls_compression_get_name(CompressionMethod algorithm)
+/**
+  * gnutls_compression_get_name - Returns a string with the name of the specified compression algorithm
+  * @algorithm: is a Compression algorithm
+  *
+  * Returns a localy allocated (with malloc) pointer to a string that contains the name 
+  * of the specified compression algorithm.
+  **/
+char *gnutls_compression_get_name(CompressionMethod algorithm)
 {
 	char *ret = NULL;
 	char *pointerTo_;
@@ -440,6 +499,30 @@ char *_gnutls_compression_get_name(CompressionMethod algorithm)
 	return ret;
 }
 
+/* return the tls number of the specified algorithm */
+int _gnutls_compression_get_num(CompressionMethod algorithm)
+{
+	int ret = -1;
+
+	/* avoid prefix */
+	GNUTLS_COMPRESSION_ALG_LOOP(ret = p->num);
+
+	return ret;
+}
+
+/* returns the gnutls internal ID of the TLS compression
+ * method num
+ */
+CompressionMethod _gnutls_compression_get_id(int num)
+{
+	CompressionMethod ret = -1;
+
+	/* avoid prefix */
+	GNUTLS_COMPRESSION_ALG_LOOP_NUM(ret = p->id);
+
+	return ret;
+}
+
 int _gnutls_compression_count()
 {
 	uint8 i, counter = 0;
@@ -484,7 +567,8 @@ _gnutls_cipher_priority(GNUTLS_STATE state, BulkCipherAlgorithm algorithm)
 	     BulkCipherAlgorithmPriority.algorithms; i++) {
 		if (state->gnutls_internals.
 		    BulkCipherAlgorithmPriority.algorithm_priority[i] ==
-		    algorithm) return i;
+		    algorithm)
+			return i;
 	}
 	return -1;
 }
@@ -515,7 +599,14 @@ int _gnutls_cipher_get_iv_size(BulkCipherAlgorithm algorithm)
 
 }
 
-char *_gnutls_cipher_get_name(BulkCipherAlgorithm algorithm)
+/**
+  * gnutls_cipher_get_name - Returns a string with the name of the specified cipher algorithm
+  * @algorithm: is an encryption algorithm
+  *
+  * Returns a localy allocated (with malloc) pointer to a string that contains the name 
+  * of the specified cipher.
+  **/
+char *gnutls_cipher_get_name(BulkCipherAlgorithm algorithm)
 {
 	char *ret = NULL;
 	char *pointerTo_;
@@ -568,9 +659,9 @@ int _gnutls_kx_server_certificate(KXAlgorithm algorithm)
 
 }
 
-MOD_AUTH_STRUCT * _gnutls_kx_auth_struct(KXAlgorithm algorithm)
+MOD_AUTH_STRUCT *_gnutls_kx_auth_struct(KXAlgorithm algorithm)
 {
-	MOD_AUTH_STRUCT * ret = NULL;
+	MOD_AUTH_STRUCT *ret = NULL;
 	GNUTLS_KX_ALG_LOOP(ret = p->auth_struct);
 	return ret;
 
@@ -592,13 +683,15 @@ inline int _gnutls_kx_priority(GNUTLS_STATE state, KXAlgorithm algorithm)
 int _gnutls_kx_server_key_exchange(KXAlgorithm algorithm)
 {
 	size_t ret = 0;
-	void* ret2=NULL;
-	
+	void *ret2 = NULL;
+
 	/* if the auth algorithm does not have a null value 
 	 * for the kx2 generation then it supports it!
 	 */
-	GNUTLS_KX_ALG_LOOP(ret2 = p->auth_struct->gnutls_generate_server_kx);
-	if (ret2!=NULL) ret = 1;
+	GNUTLS_KX_ALG_LOOP(ret2 =
+			   p->auth_struct->gnutls_generate_server_kx);
+	if (ret2 != NULL)
+		ret = 1;
 
 	return ret;
 
@@ -607,14 +700,16 @@ int _gnutls_kx_server_key_exchange(KXAlgorithm algorithm)
 int _gnutls_kx_server_key_exchange2(KXAlgorithm algorithm)
 {
 	size_t ret = 0;
-	void* ret2=NULL;
-	
+	void *ret2 = NULL;
+
 	/* if the auth algorithm does not have a null value 
 	 * for the kx2 generation then it supports it!
 	 */
-	GNUTLS_KX_ALG_LOOP(ret2 = p->auth_struct->gnutls_generate_server_kx2);
-	if (ret2!=NULL) ret = 1;
-	
+	GNUTLS_KX_ALG_LOOP(ret2 =
+			   p->auth_struct->gnutls_generate_server_kx2);
+	if (ret2 != NULL)
+		ret = 1;
+
 	return ret;
 
 }
@@ -622,27 +717,32 @@ int _gnutls_kx_server_key_exchange2(KXAlgorithm algorithm)
 int _gnutls_kx_client_key_exchange0(KXAlgorithm algorithm)
 {
 	size_t ret = 0;
-	void* ret2=NULL;
-	
+	void *ret2 = NULL;
+
 	/* if the auth algorithm does not have a null value 
 	 * for the kx0 generation then it supports it!
 	 */
-	GNUTLS_KX_ALG_LOOP(ret2 = p->auth_struct->gnutls_process_client_kx0);
-	if (ret2!=NULL) ret = 1;
-	
+	GNUTLS_KX_ALG_LOOP(ret2 =
+			   p->auth_struct->gnutls_process_client_kx0);
+	if (ret2 != NULL)
+		ret = 1;
+
 	return ret;
 
 }
+
 int _gnutls_kx_client_key_exchange(KXAlgorithm algorithm)
 {
 	size_t ret = 0;
-	void* ret2=NULL;
-	
+	void *ret2 = NULL;
+
 	/* if the auth algorithm does not have a null value 
 	 * for the kx0 generation then it supports it!
 	 */
-	GNUTLS_KX_ALG_LOOP(ret2 = p->auth_struct->gnutls_process_client_kx);
-	if (ret2!=NULL) ret = 1;
+	GNUTLS_KX_ALG_LOOP(ret2 =
+			   p->auth_struct->gnutls_process_client_kx);
+	if (ret2 != NULL)
+		ret = 1;
 	return ret;
 
 }
@@ -651,14 +751,17 @@ int _gnutls_kx_client_key_exchange(KXAlgorithm algorithm)
 int _gnutls_kx_client_cert_vrfy(KXAlgorithm algorithm)
 {
 	size_t ret = 0;
-	void* ret2=NULL;
-	
+	void *ret2 = NULL;
+
 	/* if the auth algorithm does not have a null value 
 	 * for the cert_vrfy function then it supports it!
 	 */
-	GNUTLS_KX_ALG_LOOP(ret2 = p->auth_struct->gnutls_generate_client_cert_vrfy);
-	if (ret2!=NULL) ret = 1;
-	
+	GNUTLS_KX_ALG_LOOP(ret2 =
+			   p->auth_struct->
+			   gnutls_generate_client_cert_vrfy);
+	if (ret2 != NULL)
+		ret = 1;
+
 	return ret;
 
 }
@@ -666,14 +769,17 @@ int _gnutls_kx_client_cert_vrfy(KXAlgorithm algorithm)
 int _gnutls_kx_server_cert_vrfy(KXAlgorithm algorithm)
 {
 	size_t ret = 0;
-	void* ret2=NULL;
-	
+	void *ret2 = NULL;
+
 	/* if the auth algorithm does not have a null value 
 	 * for the cert_vrfy function then it supports it!
 	 */
-	GNUTLS_KX_ALG_LOOP(ret2 = p->auth_struct->gnutls_generate_server_cert_vrfy);
-	if (ret2!=NULL) ret = 1;
-	
+	GNUTLS_KX_ALG_LOOP(ret2 =
+			   p->auth_struct->
+			   gnutls_generate_server_cert_vrfy);
+	if (ret2 != NULL)
+		ret = 1;
+
 	return ret;
 
 }
@@ -702,7 +808,14 @@ int _gnutls_kx_DH_public_value(KXAlgorithm algorithm)
 
 }
 
-char *_gnutls_kx_get_name(KXAlgorithm algorithm)
+/**
+  * gnutls_kx_get_name - Returns a string with the name of the specified key exchange algorithm
+  * @algorithm: is a key exchange algorithm
+  *
+  * Returns a localy allocated (with malloc) pointer to a string that contains the name 
+  * of the specified key exchange algorithm.
+  **/
+char *gnutls_kx_get_name(KXAlgorithm algorithm)
 {
 	char *ret = NULL;
 	char *pointerTo_;
@@ -745,22 +858,26 @@ int _gnutls_kx_is_ok(KXAlgorithm algorithm)
 	return ret;
 }
 
-int _gnutls_version_get_minor( GNUTLS_Version version) {
-int ret = -1;
+int _gnutls_version_get_minor(GNUTLS_Version version)
+{
+	int ret = -1;
 
 	GNUTLS_VERSION_ALG_LOOP(ret = p->minor);
 	return ret;
 }
 
-GNUTLS_Version _gnutls_version_get( int major, int minor) {
-int ret = -1;
+GNUTLS_Version _gnutls_version_get(int major, int minor)
+{
+	int ret = -1;
 
-	GNUTLS_VERSION_LOOP(if ((p->major == major) && (p->minor==minor)) ret = p->id);
+	GNUTLS_VERSION_LOOP(if ((p->major == major) && (p->minor == minor))
+			    ret = p->id);
 	return ret;
 }
 
-int _gnutls_version_get_major( GNUTLS_Version version) {
-int ret = -1;
+int _gnutls_version_get_major(GNUTLS_Version version)
+{
+	int ret = -1;
 
 	GNUTLS_VERSION_ALG_LOOP(ret = p->major);
 	return ret;
@@ -859,7 +976,8 @@ int _gnutls_cipher_suite_count()
 
 	for (j = 0; j < MAX_CIPHERSUITE; j++) {
 		suite.CipherSuite[0] = j;
-		if (j!=0x00 && j!=0xFF) continue;
+		if (j != 0x00 && j != 0xF6)
+			continue;
 
 		for (i = 0; i < MAX_CIPHERSUITE; i++) {
 			suite.CipherSuite[1] = i;
@@ -877,10 +995,10 @@ int _gnutls_cipher_suite_count()
 
 #define MAX_ELEM_SIZE 4
 inline
-    static int _gnutls_partition(GNUTLS_STATE state, void *_base, size_t nmemb,
-			 size_t size, int (*compar) (GNUTLS_STATE,
-						     const void *,
-						     const void *))
+    static int _gnutls_partition(GNUTLS_STATE state, void *_base,
+				 size_t nmemb, size_t size,
+				 int (*compar) (GNUTLS_STATE, const void *,
+						const void *))
 {
 	uint8 *base = _base;
 	uint8 tmp[MAX_ELEM_SIZE];
@@ -935,7 +1053,8 @@ _gnutls_qsort(GNUTLS_STATE state, void *_base, size_t nmemb, size_t size,
 		return;
 	pivot = _gnutls_partition(state, _base, nmemb, size, compar);
 
-	_gnutls_qsort(state, base, pivot<nmemb ? pivot+1 : pivot, size, compar);
+	_gnutls_qsort(state, base, pivot < nmemb ? pivot + 1 : pivot, size,
+		      compar);
 	_gnutls_qsort(state, &base[(pivot + 1) * size], nmemb - pivot - 1,
 		      size, compar);
 }
@@ -944,25 +1063,29 @@ _gnutls_qsort(GNUTLS_STATE state, void *_base, size_t nmemb, size_t size,
 /* a compare function for KX algorithms (using priorities). For use with qsort */
 static int
 _gnutls_compare_algo(GNUTLS_STATE state, const void *i_A1,
-			const void *i_A2)
+		     const void *i_A2)
 {
 	KXAlgorithm kA1 =
 	    _gnutls_cipher_suite_get_kx_algo(*(GNUTLS_CipherSuite *) i_A1);
 	KXAlgorithm kA2 =
 	    _gnutls_cipher_suite_get_kx_algo(*(GNUTLS_CipherSuite *) i_A2);
 	BulkCipherAlgorithm cA1 =
-	    _gnutls_cipher_suite_get_cipher_algo(*(GNUTLS_CipherSuite *) i_A1);
+	    _gnutls_cipher_suite_get_cipher_algo(*(GNUTLS_CipherSuite *)
+						 i_A1);
 	BulkCipherAlgorithm cA2 =
-	    _gnutls_cipher_suite_get_cipher_algo(*(GNUTLS_CipherSuite *) i_A2);
+	    _gnutls_cipher_suite_get_cipher_algo(*(GNUTLS_CipherSuite *)
+						 i_A2);
 	MACAlgorithm mA1 =
-	    _gnutls_cipher_suite_get_mac_algo(*(GNUTLS_CipherSuite *) i_A1);
+	    _gnutls_cipher_suite_get_mac_algo(*(GNUTLS_CipherSuite *)
+					      i_A1);
 	MACAlgorithm mA2 =
-	    _gnutls_cipher_suite_get_mac_algo(*(GNUTLS_CipherSuite *) i_A2);
+	    _gnutls_cipher_suite_get_mac_algo(*(GNUTLS_CipherSuite *)
+					      i_A2);
 
-	int p1 = (_gnutls_kx_priority(state, kA1)+1)*100;
-	int p2 = (_gnutls_kx_priority(state, kA2)+1)*100;
-	p1 += (_gnutls_cipher_priority(state, cA1)+1)*10;
-	p2 += (_gnutls_cipher_priority(state, cA2)+1)*10;
+	int p1 = (_gnutls_kx_priority(state, kA1) + 1) * 100;
+	int p2 = (_gnutls_kx_priority(state, kA2) + 1) * 100;
+	p1 += (_gnutls_cipher_priority(state, cA1) + 1) * 10;
+	p2 += (_gnutls_cipher_priority(state, cA2) + 1) * 10;
 	p1 += _gnutls_mac_priority(state, mA1);
 	p2 += _gnutls_mac_priority(state, mA2);
 
@@ -977,10 +1100,10 @@ _gnutls_compare_algo(GNUTLS_STATE state, const void *i_A1,
 }
 
 #if 0
-static void 
+static void
 _gnutls_bsort(GNUTLS_STATE state, void *_base, size_t nmemb,
-		  size_t size, int (*compar) (GNUTLS_STATE, const void *,
-					      const void *))
+	      size_t size, int (*compar) (GNUTLS_STATE, const void *,
+					  const void *))
 {
 	int i, j;
 	int full = nmemb * size;
@@ -1041,15 +1164,18 @@ _gnutls_supported_ciphersuites_sorted(GNUTLS_STATE state,
 		if (_gnutls_mac_priority
 		    (state,
 		     _gnutls_cipher_suite_get_mac_algo(tmp_ciphers[i])) <
-		    0) continue;
+		    0)
+			continue;
 		if (_gnutls_cipher_priority
 		    (state,
 		     _gnutls_cipher_suite_get_cipher_algo(tmp_ciphers[i]))
 		    < 0)
 			continue;
 
-		(*ciphers)[j].CipherSuite[0] = tmp_ciphers[i].CipherSuite[0];
-		(*ciphers)[j].CipherSuite[1] = tmp_ciphers[i].CipherSuite[1];
+		(*ciphers)[j].CipherSuite[0] =
+		    tmp_ciphers[i].CipherSuite[0];
+		(*ciphers)[j].CipherSuite[1] =
+		    tmp_ciphers[i].CipherSuite[1];
 		j++;
 	}
 
@@ -1083,7 +1209,7 @@ _gnutls_supported_ciphersuites(GNUTLS_STATE state,
 			       GNUTLS_CipherSuite ** ciphers)
 {
 
-	int i, ret_count, j = 0;
+	int i, ret_count, j;
 	int count = _gnutls_cipher_suite_count();
 	GNUTLS_CipherSuite *tmp_ciphers;
 
@@ -1103,7 +1229,7 @@ _gnutls_supported_ciphersuites(GNUTLS_STATE state,
 		    cs_algorithms[i].id.CipherSuite[1];
 	}
 
-	for (i = 0; i < count; i++) {
+	for (i = j = 0; i < count; i++) {
 		if (_gnutls_kx_priority
 		    (state,
 		     _gnutls_cipher_suite_get_kx_algo(tmp_ciphers[i])) < 0)
@@ -1111,15 +1237,18 @@ _gnutls_supported_ciphersuites(GNUTLS_STATE state,
 		if (_gnutls_mac_priority
 		    (state,
 		     _gnutls_cipher_suite_get_mac_algo(tmp_ciphers[i])) <
-		    0) continue;
+		    0)
+			continue;
 		if (_gnutls_cipher_priority
 		    (state,
 		     _gnutls_cipher_suite_get_cipher_algo(tmp_ciphers[i]))
 		    < 0)
 			continue;
 
-		(*ciphers)[j].CipherSuite[0] = tmp_ciphers[i].CipherSuite[0];
-		(*ciphers)[j].CipherSuite[1] = tmp_ciphers[i].CipherSuite[1];
+		(*ciphers)[j].CipherSuite[0] =
+		    tmp_ciphers[i].CipherSuite[0];
+		(*ciphers)[j].CipherSuite[1] =
+		    tmp_ciphers[i].CipherSuite[1];
 		j++;
 	}
 
@@ -1142,20 +1271,29 @@ _gnutls_supported_ciphersuites(GNUTLS_STATE state,
 
 
 /* For compression  */
+
+/* returns the TLS numbers of the compression methods we support */
 #define SUPPORTED_COMPRESSION_METHODS state->gnutls_internals.CompressionMethodPriority.algorithms
 int
-_gnutls_supported_compression_methods(GNUTLS_STATE state,
-				      uint8** comp)
+_gnutls_supported_compression_methods(GNUTLS_STATE state, uint8 ** comp)
 {
-	int i;
-	
+	int i, tmp;
+
 	*comp = gnutls_malloc(SUPPORTED_COMPRESSION_METHODS);
-	if (*comp==NULL) return GNUTLS_E_MEMORY_ERROR;
+	if (*comp == NULL)
+		return GNUTLS_E_MEMORY_ERROR;
 
 	for (i = 0; i < SUPPORTED_COMPRESSION_METHODS; i++) {
-		(*comp)[i] = (uint8)
-		    state->gnutls_internals.
-		    CompressionMethodPriority.algorithm_priority[i];
+		tmp = _gnutls_compression_get_num(state->gnutls_internals.
+						  CompressionMethodPriority.
+						  algorithm_priority[i]);
+		if (tmp == -1) {
+			gnutls_assert();
+			/* we shouldn't get here */
+			(*comp)[i] = 0;
+			continue;
+		}
+		(*comp)[i] = (uint8) tmp;
 	}
 
 	return SUPPORTED_COMPRESSION_METHODS;
