@@ -29,8 +29,9 @@
 #include "gnutls_plaintext.h"
 #include "debug.h"
 #include "gnutls_random.h"
+#include "gnutls_num.h"
 
-int _gnutls_encrypt(GNUTLS_STATE state, char *data, size_t data_size,
+int _gnutls_encrypt(GNUTLS_STATE state, const char *data, size_t data_size,
 		    uint8 ** ciphertext, ContentType type)
 {
 	GNUTLSPlaintext *gtxt;
@@ -224,8 +225,8 @@ int _gnutls_connection_state_init(GNUTLS_STATE state)
 {
 	int rc, mac_size;
 
-	state->connection_state.write_sequence_number = 0;
-	state->connection_state.read_sequence_number = 0;
+	uint64zero(&state->connection_state.write_sequence_number);
+	uint64zero(&state->connection_state.read_sequence_number);
 
 /* Update internals from CipherSuite selected.
  * If we are resuming just copy the connection state
@@ -258,6 +259,9 @@ int _gnutls_connection_state_init(GNUTLS_STATE state)
 			return rc;
 	} else {
 		 memcpy( &state->security_parameters, &state->gnutls_internals.resumed_security_parameters, sizeof(SecurityParameters));
+
+		/* clear the resumed security parameters */
+		 memset( &state->gnutls_internals.resumed_security_parameters, 0, sizeof(SecurityParameters));
 #ifdef HARD_DEBUG
 		 fprintf(stderr, "Master Secret: %s\n", _gnutls_bin2hex(state->security_parameters.master_secret, 48));
 #endif
@@ -271,13 +275,6 @@ int _gnutls_connection_state_init(GNUTLS_STATE state)
 	fprintf(stderr, "Cipher Suite: %s\n",
 		_gnutls_cipher_suite_get_name(state->
 					      gnutls_internals.current_cipher_suite));
-	fprintf(stderr, "Cipher: %s\n",
-		_gnutls_cipher_get_name(state->security_parameters.
-					bulk_cipher_algorithm));
-	fprintf(stderr, "MAC: %s\n",
-		_gnutls_mac_get_name(state->security_parameters.
-				     mac_algorithm));
-	fprintf(stderr, "Compression: %s\n", _gnutls_compression_get_name(state->security_parameters.compression_algorithm));
 #endif
 
 	if (state->connection_state.write_mac_secret!=NULL)
@@ -439,7 +436,7 @@ int _gnutls_TLSCompressed2TLSCiphertext(GNUTLS_STATE state,
 	uint8 *data;
 	uint8 pad;
 	uint8 *rand;
-	uint64 seq_num;
+	uint8* seq_num;
 	int length;
 	GNUTLS_MAC_HANDLE td;
 	int blocksize =
@@ -476,16 +473,21 @@ int _gnutls_TLSCompressed2TLSCiphertext(GNUTLS_STATE state,
 		gnutls_assert();
 		return GNUTLS_E_UNKNOWN_MAC_ALGORITHM;
 	}
-#ifdef WORDS_BIGENDIAN
-	seq_num = state->connection_state.write_sequence_number;
-	c_length = compressed->length;
-#else
-	c_length = byteswap16(compressed->length);
+
+	c_length = CONVuint16(compressed->length);
 	seq_num =
-	    byteswap64(state->connection_state.write_sequence_number);
-#endif
+	    CONVuint64(&state->connection_state.write_sequence_number);
+	if (seq_num==NULL) {
+		gnutls_free(*cipher);
+		gnutls_free(content);
+		gnutls_assert();
+		return GNUTLS_E_MEMORY_ERROR;
+	}
+
 	if (td != GNUTLS_MAC_FAILED) {	/* actually when the algorithm in not the NULL one */
-		gnutls_hmac(td, &seq_num, 8);
+		gnutls_hmac(td, seq_num, 8);
+		gnutls_free( seq_num);
+		
 		gnutls_hmac(td, &compressed->type, 1);
 		if (_gnutls_version_ssl3(state->connection_state.version) != 0) { /* TLS 1.0 only */
 			gnutls_hmac(td, &compressed->version.major, 1);
@@ -527,6 +529,10 @@ int _gnutls_TLSCompressed2TLSCiphertext(GNUTLS_STATE state,
 			rand[0] = 0;
 		} else {
 			rand[0] = (rand[0] / blocksize) * blocksize;
+			/* added to avoid the case of pad calculated 0
+			 * seen below for pad calculation.
+			 */
+			if (rand[0] > blocksize) rand[0]-=blocksize;
 		}
 
 		length =
@@ -578,7 +584,7 @@ int _gnutls_TLSCiphertext2TLSCompressed(GNUTLS_STATE state,
 	uint16 c_length;
 	uint8 *data;
 	uint8 pad;
-	uint64 seq_num;
+	uint8* seq_num;
 	uint16 length;
 	GNUTLS_MAC_HANDLE td;
 	int blocksize =
@@ -668,17 +674,19 @@ int _gnutls_TLSCiphertext2TLSCompressed(GNUTLS_STATE state,
 		return GNUTLS_E_UNKNOWN_CIPHER_TYPE;
 	}
 
-#ifdef WORDS_BIGENDIAN
-	seq_num = state->connection_state.read_sequence_number;
-	c_length = compressed->length;
-#else
-	seq_num = byteswap64(state->connection_state.read_sequence_number);
-	c_length = byteswap16((uint16) compressed->length);
-#endif
-
+	c_length = CONVuint16((uint16) compressed->length);
+	seq_num = CONVuint64( &state->connection_state.read_sequence_number);
+	if (seq_num==NULL) {
+		gnutls_free(*compress);
+		gnutls_free(content);
+		gnutls_assert();
+		return GNUTLS_E_MEMORY_ERROR;
+	}
 
 	if (td != GNUTLS_MAC_FAILED) {
-		gnutls_hmac(td, &seq_num, 8);
+		gnutls_hmac(td, seq_num, 8);
+		gnutls_free( seq_num);
+		
 		gnutls_hmac(td, &compressed->type, 1);
 		if (_gnutls_version_ssl3(state->connection_state.version) != 0) { /* TLS 1.0 only */
 			gnutls_hmac(td, &compressed->version.major, 1);
