@@ -23,6 +23,7 @@
 #include "gnutls_extensions.h"
 #include "gnutls_errors.h"
 #include "ext_srp.h"
+#include "gnutls_num.h"
 
 /* Key Exchange Section */
 #define GNUTLS_EXTENSION_ENTRY(type, ext_func_recv, ext_func_send) \
@@ -93,18 +94,14 @@ int pos=0;
 uint8 type;
 const opaque* sdata;
 int (*ext_func_recv)( GNUTLS_STATE, const opaque*, int);
-uint16 size, next1;
+uint16 size;
 
 	if (data_size < 2) return 0;
-	memcpy( &next1, data, 2);
-#ifndef WORDS_BIGENDIAN
-	next = byteswap16(next1);
-#else
-	next = next1;
-#endif
-	if (data_size < next) return 0;
-	
+
+	next = READuint16( data);
 	pos+=2;
+
+	if (data_size < next) return 0;
 	
 	do {
 		next--; if (next < 0) return 0;
@@ -112,12 +109,10 @@ uint16 size, next1;
 		pos++;
 
 		next-=2; if (next < 0) return 0;
-		memcpy( &size, &data[pos], 2);
+
+		size = READuint16(&data[pos]);
 		pos+=2;
-#ifndef WORDS_BIGENDIAN
-		size = byteswap16(size);
-#endif
-		
+				
 		sdata = &data[pos];
 		pos+=size;
 		next-=size; if (next < 0) return 0;
@@ -136,7 +131,6 @@ int _gnutls_gen_extensions( GNUTLS_STATE state, opaque** data) {
 int next, size;
 uint16 pos=0;
 opaque* sdata;
-uint16 ssize;
 int (*ext_func_send)( GNUTLS_STATE, opaque**);
 
 
@@ -152,12 +146,8 @@ int (*ext_func_send)( GNUTLS_STATE, opaque**);
 		if (size > 0) {
 			(*data) = gnutls_realloc( (*data), pos+size+3);
 			(*data)[pos++] = (uint8) next; /* set type */
-#ifndef WORDS_BIGENDIAN
-			ssize = byteswap16( (uint16)size);
-#else
-			ssize = size;
-#endif
-			memcpy( &(*data)[pos], &ssize, 2);
+
+			WRITEuint16( size, &(*data)[pos]);
 			pos+=2;
 			
 			memcpy( &(*data)[pos], sdata, size);
@@ -169,10 +159,8 @@ int (*ext_func_send)( GNUTLS_STATE, opaque**);
 
 	size = pos;
 	pos-=2; /* remove the size of the size header! */
-#ifndef WORDS_BIGENDIAN
-	pos = byteswap16(pos);
-#endif
-	memcpy( (*data), &pos, sizeof(uint16));
+
+	WRITEuint16( pos, (*data));
 
 	if (size==2) { /* empty */
 		size = 0;
