@@ -161,7 +161,6 @@ int _gnutls_read_client_hello(GNUTLS_STATE state, opaque * data,
 	int ret = 0;
 	uint16 sizeOfSuites;
 	GNUTLS_Version version;
-	time_t cur_time;
 	char *rand;
 	int len = datalen;
 	int err;
@@ -190,23 +189,19 @@ int _gnutls_read_client_hello(GNUTLS_STATE state, opaque * data,
 	pos += 2;
 
 	DECR_LEN(len, 32);
-	memmove(state->security_parameters.client_random, &data[pos], 32);
+	memcpy(state->security_parameters.client_random, &data[pos], 32);
 	pos += 32;
 
 	/* generate server random value */
-#ifdef WORDS_BIGENDIAN
-	cur_time = time(NULL);
-#else
-	cur_time = byteswap32(time(NULL));
-#endif
-	memmove(state->security_parameters.server_random, &cur_time, 4);
+	WRITEuint32( time(NULL), state->security_parameters.server_random);
+
 	rand = _gnutls_get_random(28, GNUTLS_STRONG_RANDOM);
-	memmove(&state->security_parameters.server_random[4], rand, 28);
+	memcpy(&state->security_parameters.server_random[4], rand, 28);
 	_gnutls_free_rand(rand);
 	state->security_parameters.timestamp = time(NULL);
 
 	DECR_LEN(len, 1);
-	memmove(&session_id_len, &data[pos++], 1);
+	memcpy(&session_id_len, &data[pos++], 1);
 
 	/* RESUME SESSION */
 	if (session_id_len > 32)
@@ -239,13 +234,9 @@ int _gnutls_read_client_hello(GNUTLS_STATE state, opaque * data,
 
 	/* Select a ciphersuite */
 	DECR_LEN(len, 2);
-	memmove(&sizeOfSuites, &data[pos], 2);
+	sizeOfSuites = READuint16( &data[pos]);
 	pos += 2;
 
-#ifndef WORDS_BIGENDIAN
-	sizeOfSuites = byteswap16(sizeOfSuites);
-#endif
-
 	DECR_LEN(len, sizeOfSuites);
 	ret = SelectSuite(state, state->gnutls_internals.
 			  current_cipher_suite.CipherSuite, &data[pos],
@@ -285,7 +276,7 @@ int _gnutls_read_client_hello(GNUTLS_STATE state, opaque * data,
 	}
 
 	DECR_LEN(len, 1);
-	memmove(&z, &data[pos++], 1);	/* z is the number of compression methods */
+	memcpy(&z, &data[pos++], 1);	/* z is the number of compression methods */
 
 	DECR_LEN(len, z);
 	ret = SelectCompMethod(state, &state->
@@ -432,7 +423,7 @@ static int SelectSuite(GNUTLS_STATE state, opaque ret[2], char *data,
 					_gnutls_cipher_suite_get_name(*
 								      ((GNUTLS_CipherSuite *) & data[j])));
 #endif
-				memmove(ret, &ciphers[i].CipherSuite, 2);
+				memcpy(ret, &ciphers[i].CipherSuite, 2);
 				gnutls_free(ciphers);
 
 				return 0;
@@ -461,7 +452,7 @@ static int SelectCompMethod(GNUTLS_STATE state, CompressionMethod * ret,
 	for (j = 0; j < datalen; j++) {
 		for (i = 0; i < x; i++) {
 			if (ciphers[i] == data[j]) {
-				*ret = ciphers[i];
+				*ret = _gnutls_compression_get_id(ciphers[i]);
 				gnutls_free(ciphers);
 				return 0;
 			}
@@ -482,28 +473,20 @@ int _gnutls_send_handshake(int cd, GNUTLS_STATE state, void *i_data,
 {
 	int ret;
 	uint8 *data;
-	uint24 length;
 	uint32 datasize;
 	int pos = 0;
 
-#ifdef WORDS_BIGENDIAN
 	datasize = i_datasize;
-#else
-	datasize = byteswap32(i_datasize);
-#endif
-
-	length = uint32touint24(datasize);
 
 	i_datasize += HANDSHAKE_HEADERS_SIZE;
 	data = gnutls_malloc(i_datasize);
 
-	memmove(&data[pos++], &type, 1);
-	memmove(&data[pos++], &length.pint[0], 1);
-	memmove(&data[pos++], &length.pint[1], 1);
-	memmove(&data[pos++], &length.pint[2], 1);
+	memcpy(&data[pos++], &type, 1);
+	WRITEuint24( datasize, &data[pos]);
+	pos+=3;
 
 	if (i_datasize > 4)
-		memmove(&data[pos], i_data, i_datasize - 4);
+		memcpy(&data[pos], i_data, i_datasize - 4);
 
 #ifdef HANDSHAKE_DEBUG
 	fprintf(stderr, "Handshake: %s was send [%ld bytes]\n",
@@ -536,7 +519,6 @@ int _gnutls_recv_handshake(int cd, GNUTLS_STATE state, uint8 ** data,
 	int ret;
 	uint32 length32 = 0, sum = 0;
 	uint8 *dataptr=NULL; /* for realloc */
-	uint24 num;
 	int handshake_headers = HANDSHAKE_HEADERS_SIZE;
 	HandshakeType recv_type;
 
@@ -556,6 +538,7 @@ int _gnutls_recv_handshake(int cd, GNUTLS_STATE state, uint8 ** data,
 	ret =
 	    _gnutls_Recv_int(cd, state, GNUTLS_HANDSHAKE, dataptr, SSL2_HEADERS);
 	if (ret < 0) {
+		gnutls_assert();
 		gnutls_free(dataptr);
 		return ret;
 	}
@@ -571,6 +554,7 @@ int _gnutls_recv_handshake(int cd, GNUTLS_STATE state, uint8 ** data,
 		    _gnutls_Recv_int(cd, state, GNUTLS_HANDSHAKE, &dataptr[SSL2_HEADERS],
 			     HANDSHAKE_HEADERS_SIZE-SSL2_HEADERS);
 		if (ret < 0) {
+			gnutls_assert();
 			gnutls_free(dataptr);
 			return ret;
 		}
@@ -588,14 +572,8 @@ int _gnutls_recv_handshake(int cd, GNUTLS_STATE state, uint8 ** data,
 			return GNUTLS_E_UNEXPECTED_HANDSHAKE_PACKET;
 		}
 	
-		num.pint[0] = dataptr[1];
-		num.pint[1] = dataptr[2];
-		num.pint[2] = dataptr[3];
-		length32 = uint24touint32(num);
+		length32 = READuint24( &dataptr[1]);
 
-#ifndef WORDS_BIGENDIAN
-		length32 = byteswap32(length32);
-#endif
 #ifdef HANDSHAKE_DEBUG
 		fprintf(stderr, "Handshake: %s was received [%ld bytes]\n",
 			_gnutls_handshake2str(dataptr[0]),
@@ -642,10 +620,12 @@ int _gnutls_recv_handshake(int cd, GNUTLS_STATE state, uint8 ** data,
 	ret = GNUTLS_E_UNKNOWN_ERROR;
 
 	if (length32 > 0 && data != NULL)
-		memmove(*data, &dataptr[handshake_headers], length32);
+		memcpy(*data, &dataptr[handshake_headers], length32);
 
 	/* here we buffer the handshake messages - needed at Finished message */
-	gnutls_insertHashDataBuffer(state, dataptr, length32 + handshake_headers);
+	
+	if (recv_type!=GNUTLS_HELLO_REQUEST)
+		gnutls_insertHashDataBuffer(state, dataptr, length32 + handshake_headers);
 
 	switch (recv_type) {
 	case GNUTLS_CLIENT_HELLO:
@@ -692,10 +672,38 @@ int _gnutls_recv_handshake(int cd, GNUTLS_STATE state, uint8 ** data,
 	return ret;
 }
 
-int _gnutls_send_hello_request(int cd, GNUTLS_STATE state)
+/**
+  * gnutls_send_hello_request - This function will renegotiate security parameters
+  * @cd: is a connection descriptor, as returned by socket().
+  * @state: is a a &GNUTLS_STATE structure.
+  *
+  * This function will renegotiate security parameters with the
+  * client. This should only be called in case of a server.
+  * If the client does not wish to renegotiate parameters he
+  * will reply with an alert message, thus the return code will be
+  * GNUTLS_E_WARNING_ALERT_RECEIVED and the alert will be
+  * GNUTLS_NO_RENEGOTIATION.
+  **/
+int gnutls_send_hello_request(int cd, GNUTLS_STATE state)
 {
-	return _gnutls_send_handshake(cd, state, NULL, 0,
+int 	ret;
+
+	/* only server sends that handshake packet */
+	if (state->security_parameters.entity == GNUTLS_CLIENT)
+		return GNUTLS_E_UNIMPLEMENTED_FEATURE;
+
+	ret = _gnutls_send_handshake(cd, state, NULL, 0,
 				      GNUTLS_HELLO_REQUEST);
+
+	if (ret < 0) {
+		gnutls_assert();
+		return ret;
+	}
+
+	/* begin handshake procedure again */
+	ret = gnutls_handshake(cd, state);
+
+	return ret;
 }
 
 int _gnutls_send_client_certificate(int cd, GNUTLS_STATE state)
@@ -753,12 +761,12 @@ static int _gnutls_read_server_hello( GNUTLS_STATE state, char *data, int datale
 	pos += 2;
 
 	DECR_LEN(len, 32);
-	memmove(state->security_parameters.server_random,
+	memcpy(state->security_parameters.server_random,
 		&data[pos], 32);
 	pos += 32;
 
 	DECR_LEN(len, 1);
-	memmove(&session_id_len, &data[pos++], 1);
+	memcpy(&session_id_len, &data[pos++], 1);
 
 	if (len < session_id_len) {
 		gnutls_assert();
@@ -787,10 +795,11 @@ static int _gnutls_read_server_hello( GNUTLS_STATE state, char *data, int datale
 		       resumed_security_parameters.client_random,
 		       state->security_parameters.client_random,
 		       32);
-			state->gnutls_internals.resumed = RESUME_TRUE;	/* we are resuming */
+		state->gnutls_internals.resumed = RESUME_TRUE;	/* we are resuming */
+
 		return 0;
 	} else {
-		/* keep the session id */
+		/* keep the new session id */
 		state->gnutls_internals.resumed = RESUME_FALSE;	/* we are not resuming */
 		state->security_parameters.session_id_size =
 		    session_id_len;
@@ -799,7 +808,7 @@ static int _gnutls_read_server_hello( GNUTLS_STATE state, char *data, int datale
 		}
 		pos += session_id_len;
 		DECR_LEN(len, 2);
-	memmove(&cipher_suite.CipherSuite, &data[pos], 2);
+	memcpy(&cipher_suite.CipherSuite, &data[pos], 2);
 	pos += 2;
 
 	z = 1;
@@ -816,7 +825,7 @@ static int _gnutls_read_server_hello( GNUTLS_STATE state, char *data, int datale
 		return GNUTLS_E_UNKNOWN_CIPHER_TYPE;
 	}
 
-	memmove(state->gnutls_internals.
+	memcpy(state->gnutls_internals.
 		current_cipher_suite.CipherSuite,
 		cipher_suite.CipherSuite, 2);
 
@@ -860,7 +869,7 @@ static int _gnutls_read_server_hello( GNUTLS_STATE state, char *data, int datale
 	/* move to compression */
 	z = 1;
 	DECR_LEN(len, 1);
-	memmove(&compression_method, &data[pos++], 1);
+	memcpy(&compression_method, &data[pos++], 1);
 
 	z = _gnutls_supported_compression_methods
 	    (state, &compression_methods);
@@ -872,10 +881,12 @@ static int _gnutls_read_server_hello( GNUTLS_STATE state, char *data, int datale
 		}
 	}
 
-	if (z != 0)
+	if (z != 0) {
+		gnutls_assert();
 		return GNUTLS_E_UNKNOWN_COMPRESSION_ALGORITHM;
-	memmove(&state->gnutls_internals.compression_method,
-		&compression_method, 1);
+	}
+	state->gnutls_internals.compression_method =
+		_gnutls_compression_get_id( compression_method);
 
 	gnutls_free(cipher_suites);
 	gnutls_free(compression_methods);
@@ -889,28 +900,25 @@ static int _gnutls_read_server_hello( GNUTLS_STATE state, char *data, int datale
 	return ret;
 }
 
-int _gnutls_send_hello(int cd, GNUTLS_STATE state, opaque * SessionID,
-		       uint8 SessionIDLen)
+int _gnutls_send_hello(int cd, GNUTLS_STATE state)
 {
 	char *rand;
 	char *data = NULL;
 	opaque *extdata;
 	int extdatalen;
-	uint8 session_id_len, z;
-	uint32 cur_time;
+	uint8 z;
 	int pos = 0;
 	GNUTLS_CipherSuite *cipher_suites;
 	uint8 *compression_methods;
 	int i, datalen, ret = 0;
 	uint16 x;
 
-	session_id_len = SessionIDLen;
-	if (SessionID == NULL)
-		session_id_len = 0;
-
-
 	if (state->security_parameters.entity == GNUTLS_CLIENT) {
+		opaque * SessionID = state->gnutls_internals.resumed_security_parameters.session_id;
+		uint8 session_id_len = state->gnutls_internals.resumed_security_parameters.session_id_size;
 
+		if (SessionID==NULL) session_id_len = 0;
+		
 		datalen = 2 + 4 + (session_id_len + 1) + 28 + 3;
 		/* 2 for version, 4 for unix time, 28 for random bytes 2 for cipher suite's
 		 * size and 1 for compression method's size 
@@ -923,49 +931,39 @@ int _gnutls_send_hello(int cd, GNUTLS_STATE state, opaque * SessionID,
 		data[pos++] =
 		    _gnutls_version_get_minor(state->connection_state.
 					      version);
-#ifdef WORDS_BIGENDIAN
-		cur_time = time(NULL);
-#else
-		cur_time = byteswap32(time(NULL));
-#endif
-		memmove(state->security_parameters.client_random,
-			&cur_time, 4);
+
+		WRITEuint32( time(NULL), state->security_parameters.client_random);
 
 		rand = _gnutls_get_random(28, GNUTLS_STRONG_RANDOM);
-		memmove(&state->security_parameters.client_random[4], rand,
+		memcpy(&state->security_parameters.client_random[4], rand,
 			28);
 		_gnutls_free_rand(rand);
 
 		state->security_parameters.timestamp = time(0);
 
-		memmove(&data[pos],
+		memcpy(&data[pos],
 			state->security_parameters.client_random, 32);
 		pos += 32;
 
-		memmove(&data[pos++], &session_id_len, 1);
+		memcpy(&data[pos++], &session_id_len, 1);
 
 		if (session_id_len > 0) {
-			memmove(&data[pos], SessionID, session_id_len);
+			memcpy(&data[pos], SessionID, session_id_len);
 		}
 		pos += session_id_len;
 
 		x = _gnutls_supported_ciphersuites_sorted(state,
 							  &cipher_suites);
 		x *= sizeof(uint16);	/* in order to get bytes */
-#ifdef WORDS_BIGENDIAN
-		memmove(&data[pos], &x, sizeof(uint16));
-#else
-		x = byteswap16(x);
-		memmove(&data[pos], &x, sizeof(uint16));
-		x = byteswap16(x);
-#endif
+
+		WRITEuint16( x, &data[pos]);
 		pos += sizeof(uint16);
 
 		datalen += x;
 		data = gnutls_realloc(data, datalen);
 
 		for (i = 0; i < x / 2; i++) {
-			memmove(&data[pos], &cipher_suites[i].CipherSuite,
+			memcpy(&data[pos], &cipher_suites[i].CipherSuite,
 				2);
 			pos += 2;
 		}
@@ -974,13 +972,13 @@ int _gnutls_send_hello(int cd, GNUTLS_STATE state, opaque * SessionID,
 		z = _gnutls_supported_compression_methods
 		    (state, &compression_methods);
 
-		memmove(&data[pos++], &z, 1);	/* put the number of compression methods */
+		memcpy(&data[pos++], &z, 1);	/* put the number of compression methods */
 
 		datalen += z;
 		data = gnutls_realloc(data, datalen);
 
 		for (i = 0; i < z; i++) {
-			memmove(&data[pos++], &compression_methods[i], 1);
+			memcpy(&data[pos++], &compression_methods[i], 1);
 		}
 
 		gnutls_free(compression_methods);
@@ -1000,6 +998,12 @@ int _gnutls_send_hello(int cd, GNUTLS_STATE state, opaque * SessionID,
 
 
 	} else {		/* SERVER */
+		uint8 comp;
+		opaque * SessionID = state->security_parameters.session_id;
+		uint8 session_id_len = state->security_parameters.session_id_size;
+		
+		if (SessionID==NULL) session_id_len = 0;
+		
 		datalen = 2 + session_id_len + 1 + 32;
 		data = gnutls_malloc(datalen);
 
@@ -1010,27 +1014,28 @@ int _gnutls_send_hello(int cd, GNUTLS_STATE state, opaque * SessionID,
 		    _gnutls_version_get_minor(state->connection_state.
 					      version);
 
-		memmove(&data[pos],
+		memcpy(&data[pos],
 			state->security_parameters.server_random, 32);
 		pos += 32;
 
-		memmove(&data[pos++], &session_id_len, sizeof(uint8));
+		memcpy(&data[pos++], &session_id_len, sizeof(uint8));
 		if (session_id_len > 0) {
-			memmove(&data[pos], SessionID, session_id_len);
+			memcpy(&data[pos], SessionID, session_id_len);
 		}
 		pos += session_id_len;
 
 		datalen += 2;
 		data = gnutls_realloc(data, datalen);
-		memmove(&data[pos],
+		memcpy(&data[pos],
 			&state->gnutls_internals.
 			current_cipher_suite.CipherSuite, 2);
 		pos += 2;
 
 		datalen += 1;
 		data = gnutls_realloc(data, datalen);
-		memmove(&data[pos++],
-			&state->gnutls_internals.compression_method, 1);
+		
+		comp = (uint8) _gnutls_compression_get_num(state->gnutls_internals.compression_method);
+		memcpy(&data[pos++], &comp, 1);
 
 		extdatalen = _gnutls_gen_extensions(state, &extdata);
 		if (extdatalen > 0) {
@@ -1085,7 +1090,6 @@ int _gnutls_recv_certificate(int cd, GNUTLS_STATE state, char *data,
 	char *certificate_list;
 	int ret = 0;
 	uint32 sizeOfCert;
-	uint24 num;
 
 	if (state->security_parameters.entity == GNUTLS_CLIENT) {
 		if (datalen < 2) {
@@ -1093,22 +1097,16 @@ int _gnutls_recv_certificate(int cd, GNUTLS_STATE state, char *data,
 			return GNUTLS_E_UNEXPECTED_PACKET_LENGTH;
 		}
 
-		num.pint[0] = data[pos];
-		num.pint[1] = data[pos + 1];
-		num.pint[2] = data[pos + 2];
-		sizeOfCert = uint24touint32(num);
-
+		sizeOfCert = READuint24( &data[pos]);
 		pos += 3;
-#ifndef WORDS_BIGENDIAN
-		sizeOfCert = byteswap32(sizeOfCert);
-#endif
+
 		if (sizeOfCert > MAX24) {
 			gnutls_assert();
 			return GNUTLS_E_UNEXPECTED_PACKET_LENGTH;
 		}
 		certificate_list = gnutls_malloc(sizeOfCert);
 
-		memmove(certificate_list, &data[pos], sizeOfCert);
+		memcpy(certificate_list, &data[pos], sizeOfCert);
 
 		/* Verify certificates !!! */
 
@@ -1128,25 +1126,44 @@ int _gnutls_recv_certificate(int cd, GNUTLS_STATE state, char *data,
 }
 
 
-/* This is the main function in the handshake protocol. This does actually
- * everything. (exchange hello messages etc).
- */
+/**
+  * gnutls_handshake - This the main function in the handshake protocol.
+  * @cd: is a connection descriptor, as returned by socket().
+  * @state: is a a &GNUTLS_STATE structure.
+  *
+  * This function does the handshake of the TLS/SSL protocol,
+  * and initializes the TLS connection. Here the identity of the peer
+  * is checked automatically.
+  * This function will fail if any problem is encountered,
+  * and the connection should be terminated.
+  **/
 int gnutls_handshake(int cd, GNUTLS_STATE state)
 {
 	int ret;
 
 	ret = gnutls_handshake_begin(cd, state);
 	/* FIXME: check certificate */
+
 	if (ret == 0)
 		ret = gnutls_handshake_finish(cd, state);
 
 	return ret;
 }
 
-/* in this function we initiate the handshake and we receive - 
- * if requested - the certificate. This certificate should be checked
- * somehow.
- */
+/**
+  * gnutls_handshake_begin - This function does a partial handshake of the TLS/SSL protocol.
+  * @cd: is a connection descriptor, as returned by socket().
+  * @state: is a a &GNUTLS_STATE structure.
+  *
+  * This function initiates the handshake of the TLS/SSL protocol.
+  * Here we will receive - if requested and supported by the ciphersuite -
+  * the peer's certificate. 
+  *
+  * This function will fail if any problem in the handshake is encountered.   
+  * However this failure will not be fatal. However you may choose to
+  * continue the handshake - eg. even if the certificate cannot
+  * be verified- by calling gnutls_handshake_finish().
+  **/
 int gnutls_handshake_begin(int cd, GNUTLS_STATE state)
 {
 	int ret;
@@ -1164,13 +1181,7 @@ int gnutls_handshake_begin(int cd, GNUTLS_STATE state)
 						session_id_size));
 #endif
 		ret =
-		    _gnutls_send_hello(cd, state,
-				       state->gnutls_internals.
-				       resumed_security_parameters.
-				       session_id,
-				       state->gnutls_internals.
-				       resumed_security_parameters.
-				       session_id_size);
+		    _gnutls_send_hello(cd, state);
 		if (ret < 0) {
 			ERR("send hello", ret);
 			gnutls_clearHashDataBuffer(state);
@@ -1205,18 +1216,16 @@ int gnutls_handshake_begin(int cd, GNUTLS_STATE state)
 					   GNUTLS_CLIENT_HELLO);
 		if (ret < 0) {
 			ERR("recv hello", ret);
+			gnutls_assert();
 			gnutls_clearHashDataBuffer(state);
 			return ret;
 		}
 
 		ret =
-		    _gnutls_send_hello(cd, state,
-				       state->security_parameters.
-				       session_id,
-				       state->security_parameters.
-				       session_id_size);
+		    _gnutls_send_hello(cd, state);
 		if (ret < 0) {
 			ERR("send hello", ret);
+			gnutls_assert();
 			gnutls_clearHashDataBuffer(state);
 			return ret;
 		}
@@ -1231,6 +1240,7 @@ int gnutls_handshake_begin(int cd, GNUTLS_STATE state)
 			ret = _gnutls_send_server_kx_message(cd, state);
 		if (ret < 0) {
 			ERR("send server kx", ret);
+			gnutls_assert();
 			gnutls_clearHashDataBuffer(state);
 			return ret;
 		}
@@ -1327,9 +1337,16 @@ static int _gnutls_recv_handshake_final(int cd, GNUTLS_STATE state,
 	return ret;
 }
 
-/* in this function we finish the handshake procedure
- * This should happen only if we trust the peer. (check certificate)
- */
+/**
+  * gnutls_handshake_finish - This function finished a partial handshake of the TLS/SSL protocol.
+  * @cd: is a connection descriptor, as returned by socket().
+  * @state: is a a &GNUTLS_STATE structure.
+  *
+  * This function does the final stuff of the handshake protocol.
+  * You should call it only if you used gnutls_handshake_begin() and
+  * you have somehow verified the identity of the peer.
+  * This function will fail if any problem is encountered.
+  **/
 int gnutls_handshake_finish(int cd, GNUTLS_STATE state)
 {
 	int ret = 0;
@@ -1455,6 +1472,8 @@ int gnutls_handshake_finish(int cd, GNUTLS_STATE state)
 		if (ret < 0) {
 			gnutls_assert();
 			gnutls_clearHashDataBuffer(state);
+			/* for srp */
+			if (state->security_parameters.kx_algorithm==GNUTLS_KX_SRP) return GNUTLS_E_AUTH_FAILED;
 			return ret;
 		}
 
@@ -1477,6 +1496,8 @@ int gnutls_handshake_finish(int cd, GNUTLS_STATE state)
 		if (ret < 0) {
 			gnutls_assert();
 			gnutls_clearHashDataBuffer(state);
+			/* in srp failure here - means authentication error */
+			if (state->security_parameters.kx_algorithm==GNUTLS_KX_SRP) return GNUTLS_E_AUTH_FAILED;
 			return ret;
 		}
 	}
@@ -1497,7 +1518,7 @@ int _gnutls_generate_session_id(char *session_id, uint8 * len)
 	char *rand;
 	rand = _gnutls_get_random(32, GNUTLS_WEAK_RANDOM);
 
-	memmove(session_id, rand, 32);
+	memcpy(session_id, rand, 32);
 	_gnutls_free_rand(rand);
 	*len = 32;
 
@@ -1507,3 +1528,41 @@ int _gnutls_generate_session_id(char *session_id, uint8 * len)
 #endif
 	return 0;
 }
+
+int _gnutls_recv_hello_request(int cd, GNUTLS_STATE state, void* data, uint32 data_size) {
+int ret;
+
+	/* only client should receive that */
+	if (state->security_parameters.entity == GNUTLS_SERVER)
+		return GNUTLS_E_UNEXPECTED_PACKET;
+
+	/* just return an alert that we don't like that */
+	ret = _gnutls_send_alert( cd, state, GNUTLS_WARNING, GNUTLS_NO_RENEGOTIATION);
+	if (ret < 0) {
+		gnutls_assert();
+		return ret;
+	}
+	return 0;
+
+#if 0 /* this does not work - yet */
+uint8 type;
+
+	if (state->security_parameters.entity == GNUTLS_SERVER) {
+		gnutls_assert();
+		return GNUTLS_E_UNEXPECTED_PACKET;
+	}
+	
+	if (data_size < 1) {
+		gnutls_assert();
+		return GNUTLS_E_UNEXPECTED_PACKET_LENGTH;
+	}
+	
+	type = ((uint8*)data)[0];
+	if (type==GNUTLS_HELLO_REQUEST)
+		return gnutls_handshake( cd, state);
+	else {
+		gnutls_assert();
+		return GNUTLS_E_UNEXPECTED_PACKET;
+	}
+#endif
+}
