@@ -40,6 +40,31 @@
 
 #define MAX(X,Y) (X >= Y ? X : Y);
 
+static int print_info( GNUTLS_STATE state) {
+char *tmp;
+const DH_ANON_AUTH_INFO *dh_info;
+
+	tmp = gnutls_kx_get_name(gnutls_get_current_kx( state));
+	printf("- Key Exchange: %s\n", tmp); free(tmp);
+	if (gnutls_get_current_kx(state) == GNUTLS_KX_DH_ANON) {
+		dh_info = gnutls_get_auth_info(state);
+		if (dh_info != NULL)
+			printf("- Anonymous DH using prime of %d bits\n",
+			       dh_info->bits);
+	}
+
+	tmp = gnutls_compression_get_name(gnutls_get_current_compression_method( state));
+	printf("- Compression: %s\n", tmp); free(tmp);
+
+	tmp = gnutls_cipher_get_name(gnutls_get_current_cipher( state));
+	printf("- Cipher: %s\n", tmp); free(tmp);
+
+	tmp = gnutls_mac_get_name(gnutls_get_current_mac_algorithm( state));
+	printf("- MAC: %s\n", tmp); free(tmp);
+
+	return 0;
+}
+
 int main()
 {
 	int err, ret;
@@ -57,11 +82,10 @@ int main()
 	int maxfd;
 	struct timeval tv;
 	int user_term = 0;
-	char *tmp;
 	SRP_CLIENT_CREDENTIALS cred;
 	
-	cred.username = "test2";
-	cred.password = "test2";
+	cred.username = "test";
+	cred.password = "test";
 	
 //	signal(SIGPIPE, SIG_IGN);
 
@@ -80,22 +104,22 @@ int main()
 	gnutls_init(&state, GNUTLS_CLIENT);
 	gnutls_set_current_version( state, GNUTLS_TLS1);
 
-	gnutls_set_cipher_priority( state, 3, GNUTLS_3DES, GNUTLS_ARCFOUR, GNUTLS_RIJNDAEL);
-	gnutls_set_compression_priority( state, 2, GNUTLS_ZLIB, GNUTLS_NULL_COMPRESSION);
-	gnutls_set_kx_priority( state, 2, GNUTLS_KX_SRP, GNUTLS_KX_ANON_DH);
-	gnutls_set_kx_cred( state, GNUTLS_KX_ANON_DH, NULL);
+	gnutls_set_cipher_priority( state, GNUTLS_3DES, GNUTLS_ARCFOUR, GNUTLS_RIJNDAEL, 0);
+	gnutls_set_compression_priority( state, GNUTLS_ZLIB, GNUTLS_NULL_COMPRESSION, 0);
+	gnutls_set_kx_priority( state, GNUTLS_KX_SRP, GNUTLS_KX_DH_ANON, 0);
+	gnutls_set_kx_cred( state, GNUTLS_KX_DH_ANON, NULL);
 	gnutls_set_kx_cred( state, GNUTLS_KX_SRP, &cred);
 
-	gnutls_set_mac_priority( state, 2, GNUTLS_MAC_SHA, GNUTLS_MAC_MD5);
+	gnutls_set_mac_priority( state, GNUTLS_MAC_SHA, GNUTLS_MAC_MD5, 0);
 	ret = gnutls_handshake(sd, state);
 
 	if (ret < 0) {
-		fprintf(stderr, "- Handshake has failed\n");
+		fprintf(stderr, "*** Handshake has failed\n");
 		gnutls_perror(ret);
-		gnutls_deinit(&state);
+		gnutls_deinit(state);
 		return 1;
 	} else {
-		fprintf(stderr, "- Handshake was completed\n");
+		printf("- Handshake was completed\n");
 	}
 	gnutls_get_current_session( state, NULL, &session_size);
 	session = malloc(session_size);
@@ -105,13 +129,16 @@ int main()
 	session_id = malloc(session_id_size);
 	gnutls_get_current_session_id( state, session_id, &session_id_size);
 
-	fprintf(stderr, "- Disconnecting\n");
+/* print some information */
+	print_info( state);
+
+	printf("- Disconnecting\n");
 	gnutls_close(sd, state);
 	shutdown( sd, SHUT_WR);
 	close(sd);	
-	gnutls_deinit( &state);	
+	gnutls_deinit( state);	
 	
-	fprintf(stderr, "\n\n- Connecting again- trying to resume previous session\n");
+	printf("\n\n- Connecting again- trying to resume previous session\n");
 	sd = socket(AF_INET, SOCK_STREAM, 0);
 	ERR(sd, "socket");
 
@@ -125,13 +152,13 @@ int main()
 	
 	gnutls_set_current_version( state, GNUTLS_TLS1);
 
-	gnutls_set_cipher_priority( state, 4, GNUTLS_3DES, GNUTLS_TWOFISH , GNUTLS_RIJNDAEL, GNUTLS_ARCFOUR);
-	gnutls_set_compression_priority( state, 1, GNUTLS_NULL_COMPRESSION);
-	gnutls_set_kx_priority( state, 2, GNUTLS_KX_SRP, GNUTLS_KX_ANON_DH);
-	gnutls_set_kx_cred( state, GNUTLS_KX_ANON_DH, NULL);
+	gnutls_set_cipher_priority( state, GNUTLS_3DES, GNUTLS_TWOFISH , GNUTLS_RIJNDAEL, GNUTLS_ARCFOUR, 0);
+	gnutls_set_compression_priority( state, GNUTLS_NULL_COMPRESSION, 0);
+	gnutls_set_kx_priority( state, GNUTLS_KX_SRP, GNUTLS_KX_DH_ANON, 0);
+	gnutls_set_kx_cred( state, GNUTLS_KX_DH_ANON, NULL);
 	gnutls_set_kx_cred( state, GNUTLS_KX_SRP, &cred);
 
-	gnutls_set_mac_priority( state, 2, GNUTLS_MAC_SHA, GNUTLS_MAC_MD5);
+	gnutls_set_mac_priority( state, GNUTLS_MAC_SHA, GNUTLS_MAC_MD5, 0);
 
 #ifdef RESUME
 	gnutls_set_current_session( state, session, session_size);
@@ -141,12 +168,12 @@ int main()
 	ret = gnutls_handshake(sd, state);
 
 	if (ret < 0) {
-		fprintf(stderr, "- Handshake failed\n");
+		fprintf(stderr, "*** Handshake failed\n");
 		gnutls_perror(ret);
-		gnutls_deinit(&state);
+		gnutls_deinit(state);
 		return 1;
 	} else {
-		fprintf(stderr, "- Handshake was completed\n");
+		printf("- Handshake was completed\n");
 	}
 
 	/* check if we actually resumed the previous session */
@@ -154,27 +181,17 @@ int main()
 	tmp_session_id = malloc(tmp_session_id_size);
 	gnutls_get_current_session_id( state, tmp_session_id, &tmp_session_id_size);
 	if (memcmp( tmp_session_id, session_id, session_id_size)==0) {
-		fprintf(stderr, "- Previous session was resumed\n");
+		printf("- Previous session was resumed\n");
 	} else {
-		fprintf(stderr, "- Previous session was NOT resumed\n");	
+		fprintf(stderr, "*** Previous session was NOT resumed\n");	
 	}
 	free(tmp_session_id);
 	free(session_id);
 
 /* print some information */
-	tmp = _gnutls_kx_get_name(gnutls_get_current_kx( state));
-	printf("Key Exchange: %s\n", tmp); free(tmp);
-
-	tmp = _gnutls_compression_get_name(gnutls_get_current_compression_method( state));
-	printf("Compression: %s\n", tmp); free(tmp);
-
-	tmp = _gnutls_cipher_get_name(gnutls_get_current_cipher( state));
-	printf("Cipher: %s\n", tmp); free(tmp);
-
-	tmp = _gnutls_mac_get_name(gnutls_get_current_mac_algorithm( state));
-	printf("MAC: %s\n", tmp); free(tmp);
-
-	printf("\nSimple Client Mode:\n\n");
+	print_info( state);
+	
+	printf("\n- Simple Client Mode:\n\n");
 
 	FD_ZERO(&rset);
 	for(;;) {
@@ -194,20 +211,26 @@ int main()
 
 			if (gnutls_is_fatal_error(ret) == 1) {
 				if (ret == GNUTLS_E_CLOSURE_ALERT_RECEIVED || ret == GNUTLS_E_INVALID_SESSION) {
-					fprintf(stderr,
-						"- Peer has closed the GNUTLS connection\n");
+					printf("- Peer has closed the GNUTLS connection\n");
 					break;
 				} else {
-					fprintf(stderr, "- Received corrupted data(%d) - server has terminated the connection abnormally\n",
+					fprintf(stderr, "*** Received corrupted data(%d) - server has terminated the connection abnormally\n",
 						ret);
 					break;
 				}
 			} else {
-				fprintf(stdout, "- Received[%d]: ", ret);
-				for (ii=0;ii<ret;ii++) {
-					fputc(buffer[ii], stdout);
+				if (ret==GNUTLS_E_WARNING_ALERT_RECEIVED || ret==GNUTLS_E_FATAL_ALERT_RECEIVED)
+					printf("* Received alert [%d]\n", gnutls_get_last_alert(state));
+				if (ret==GNUTLS_E_GOT_HELLO_REQUEST)
+					printf("* Received HelloRequest message\n");
+
+				if (ret > 0) {
+					printf("- Received[%d]: ", ret);
+					for (ii=0;ii<ret;ii++) {
+						fputc(buffer[ii], stdout);
+					}
+					fputs("\n", stdout);
 				}
-				fputs("\n", stdout);
 			}
 			if (user_term!=0) break;
 		}
@@ -219,7 +242,7 @@ int main()
 				continue;
 			}
 			gnutls_write( sd, state, buffer, strlen(buffer));
-			fprintf(stdout, "- Sent: %d bytes\n", strlen(buffer));
+			printf("- Sent: %d bytes\n", strlen(buffer));
 		}
 	}
 	if (user_term!=0) gnutls_close(sd, state);
@@ -227,6 +250,6 @@ int main()
 	shutdown( sd, SHUT_RDWR); /* no more receptions */
 	close(sd);
 	
-	gnutls_deinit(&state);
+	gnutls_deinit( state);
 	return 0;
 }
