@@ -27,29 +27,32 @@
 #include "../lib/gnutls_srp.h"
 #include "../lib/crypt.h"
 #include "../lib/cert_b64.h"
-#include "gaa.h"
+#include "crypt-gaa.h"
 #include <pwd.h>
 #include <sys/types.h>
 #include <unistd.h>
 
-int crypt_int(char *username, char *passwd, int crypt, int salt,
-	      char *tpasswd_conf, char *tpasswd);
-static int read_conf_values(MPI * g, MPI * n, char *str, int str_size);
+#define _MAX(x,y) x>y?x:y
 
-int generate_create_conf(char *file)
+int crypt_int(char *username, char *passwd, int crypt, int salt,
+	      char *tpasswd_conf, char *tpasswd, int uindex);
+static int read_conf_values(MPI * g, MPI * n, char *str, int str_size);
+static int _verify_passwd_int(char* username, char* passwd, char* salt, MPI g, MPI n);
+
+int generate_create_conf(char *tpasswd_conf, int bits)
 {
 	FILE *fd;
 	char line[5 * 1024];
 	int index = 1;
 	unsigned char *g, *n;
 
-	fd = fopen(file, "w");
+	fd = fopen(tpasswd_conf, "w");
 	if (fd == NULL) {
-		fprintf(stderr, "Cannot open file '%s'\n", file);
+		fprintf(stderr, "Cannot open file '%s'\n", tpasswd_conf);
 		return -1;
 	}
 
-	_gnutls_srp_gn(&g, &n);
+	_gnutls_srp_gn(&g, &n, bits);
 	sprintf(line, "%d:%s:%s\n", index, n, g);
 
 	fwrite(line, 1, strlen(line), fd);
@@ -59,35 +62,59 @@ int generate_create_conf(char *file)
 
 }
 
+static int _verify_passwd_int(char* username, char* passwd, char* salt, MPI g, MPI n) {
+	if (salt==NULL) return -1;
 
-int verify_passwd(char *conffile, char *file, char *username, char *passwd)
-{
-	FILE *fd;
-	char line[5 * 1024];
-	int i;
-	MPI g, n;
-	int index;
-	char *p;
-
-	fd = fopen(conffile, "r");
-	if (fd == NULL) {
-		fprintf(stderr, "Cannot find %s\n", conffile);
-		return -1;
+	if (gnutls_crypt_vrfy
+	    (username, passwd, salt, g, n) == 0) {
+		fprintf(stderr, "Password verified\n");
+		return 0;
+	} else {
+		fprintf(stderr,
+			"Password does NOT match\n");
 	}
+	return -1;
+}
 
-	p = fgets(line, sizeof(line) - 1, fd);
+static int filecopy( char* src, char* dst) {
+FILE *fd, *fd2;
+char line[5*1024];
+char *p;
 
-	if (p == NULL) {
-		fprintf(stderr, "Cannot find entry in %s\n", conffile);
-		return -1;
-	}
-	line[sizeof(line) - 1] = 0;
+		fd = fopen(dst, "w");
+		if (fd == NULL) {
+			fprintf(stderr, "Cannot open '%s' for write\n",
+				dst);
+			return -1;
+		}
 
-	fclose(fd);
-	if ((index = read_conf_values(&g, &n, line, strlen(line))) < 0) {
-		fprintf(stderr, "Cannot parse conf file '%s'\n", conffile);
-		return -1;
-	}
+		fd2 = fopen(src, "r");
+		if (fd2 == NULL) {
+			/* empty file */
+			fclose(fd);
+			return 0;
+		}
+
+		line[sizeof(line)-1] = 0;
+		do {
+			p = fgets( line, sizeof(line)-1, fd2);
+			if (p==NULL) break;
+			
+			fputs( line, fd);
+		} while(1);
+
+		fclose(fd);
+		fclose(fd2);
+
+		return 0;	
+}
+
+/* accepts password file */
+static int find_index(char* username, char* file) {
+FILE * fd;
+char *pos;
+char line[5*1024];
+int i;
 
 	fd = fopen(file, "r");
 	if (fd == NULL) {
@@ -102,15 +129,80 @@ int verify_passwd(char *conffile, char *file, char *username, char *passwd)
 		       && (i < sizeof(line))) {
 			i++;
 		}
-		if (strncmp(username, line, (i>strlen(username))?i:strlen(username)) == 0) {
-			if (gnutls_crypt_vrfy
-			    (username, passwd, &line[++i], g, n) == 0) {
-				fprintf(stderr, "Password verified\n");
-			} else {
-				fprintf(stderr,
-					"Password does NOT match\n");
+		if (strncmp(username, line, _MAX(i,strlen(username)) )  == 0) {
+			/* find the index */
+			pos = rindex(line, ':');
+			pos++;
+			fclose(fd);
+			return atoi(pos);
+		}
+	}
+
+	fclose(fd);
+	return -1;
+}
+
+int verify_passwd(char *conffile, char *tpasswd, char *username, char *passwd)
+{
+	FILE *fd;
+	char line[5 * 1024];
+	int i;
+	MPI g, n;
+	int iindex;
+	char *p, *pos;
+
+	iindex = find_index( username, tpasswd);
+	if (iindex==-1) {
+		fprintf(stderr, "Cannot find '%s' in %s\n", username, tpasswd);
+		return -1;
+	}
+	
+	fd = fopen(conffile, "r");
+	if (fd == NULL) {
+		fprintf(stderr, "Cannot find %s\n", conffile);
+		return -1;
+	}
+
+	do {
+		p = fgets(line, sizeof(line) - 1, fd);
+	} while(p!=NULL && atoi(p)!=iindex);
+	
+	if (p == NULL) {
+		fprintf(stderr, "Cannot find entry in %s\n", conffile);
+		return -1;
+	}
+	line[sizeof(line) - 1] = 0;
+
+	fclose(fd);
+
+	if ((iindex = read_conf_values(&g, &n, line, strlen(line))) < 0) {
+		fprintf(stderr, "Cannot parse conf file '%s'\n", conffile);
+		return -1;
+	}
+
+
+	fd = fopen(tpasswd, "r");
+	if (fd == NULL) {
+		fprintf(stderr, "Cannot open file '%s'\n", tpasswd);
+		return -1;
+	}
+
+	while (fgets(line, sizeof(line), fd) != NULL) {
+		/* move to first ':' */
+		i = 0;
+		while ((line[i] != ':') && (line[i] != '\0')
+		       && (i < sizeof(line))) {
+			i++;
+		}
+		if (strncmp(username, line, _MAX(i,strlen(username)) )  == 0) {
+			pos = index(line, ':');
+			fclose(fd);
+			if (pos==NULL) {
+				fprintf(stderr, "Cannot parse conf file '%s'\n", conffile);
+				return -1;
 			}
-			return 0;
+			pos++;
+			return _verify_passwd_int( username, passwd, pos, g, n);
 		}
 	}
 
@@ -137,7 +229,7 @@ int main(int argc, char **argv)
 	salt = info.salt;
 
 	if (info.create_conf != NULL) {
-		return generate_create_conf(info.create_conf);
+		return generate_create_conf(info.create_conf, info.bits);
 	}
 
 	if (info.passwd == NULL)
@@ -184,19 +276,20 @@ int main(int argc, char **argv)
 
 
 	return crypt_int(info.username, passwd, crypt, salt,
-			 info.passwd_conf, info.passwd);
+			 info.passwd_conf, info.passwd, info.index);
 
 }
 
 int crypt_int(char *username, char *passwd, int crypt, int salt,
-	      char *tpasswd_conf, char *tpasswd)
+	      char *tpasswd_conf, char *tpasswd, int uindex)
 {
 	FILE *fd;
 	char *cr;
 	MPI g, n;
 	char line[5 * 1024];
-	char *p;
-	int index;
+	char *p, *pp;
+	int iindex;
+	char tmpname[1024];
 
 	fd = fopen(tpasswd_conf, "r");
 	if (fd == NULL) {
@@ -204,7 +297,10 @@ int crypt_int(char *username, char *passwd, int crypt, int salt,
 		return -1;
 	}
 
-	p = fgets(line, sizeof(line) - 1, fd);
+	do { /* find the specified uindex in file */
+		p = fgets(line, sizeof(line) - 1, fd);
+		iindex = atoi(p);
+	} while( p!=NULL && iindex!=uindex);
 
 	if (p == NULL) {
 		fprintf(stderr, "Cannot find entry in %s\n", tpasswd_conf);
@@ -213,7 +309,7 @@ int crypt_int(char *username, char *passwd, int crypt, int salt,
 	line[sizeof(line) - 1] = 0;
 
 	fclose(fd);
-	if ((index = read_conf_values(&g, &n, line, strlen(line))) < 0) {
+	if ((iindex = read_conf_values(&g, &n, line, strlen(line))) < 0) {
 		fprintf(stderr, "Cannot parse conf file '%s'\n",
 			tpasswd_conf);
 		return -1;
@@ -224,16 +320,71 @@ int crypt_int(char *username, char *passwd, int crypt, int salt,
 		fprintf(stderr, "Cannot gnutls_crypt()...\n");
 		return -1;
 	} else {
+		/* delete previous entry */
+		struct stat st;
+		FILE * fd2;
+		int put;
 
-		fd = fopen(tpasswd, "a");
-		if (fd == NULL) {
-			fprintf(stderr, "Cannot open '%s' for append\n",
-				tpasswd);
+		if (strlen(tpasswd) > sizeof(tmpname)+5) {
+			fprintf(stderr, "file '%s' is tooooo long\n", tpasswd);
 			return -1;
 		}
-		fprintf(fd, "%s:%s:%u\n", username, cr, index);
-		fclose(fd);
+		strcpy( tmpname, tpasswd);
+		strcat( tmpname, ".tmp");
+
+		if ( stat( tmpname, &st) != -1) {
+			fprintf(stderr, "file '%s' is locked\n", tpasswd);
+			return -1;
+		}
+
+		if (filecopy( tpasswd, tmpname)!=0) {
+			fprintf(stderr, "Cannot copy '%s' to '%s'\n",
+				tpasswd, tmpname);
+			return -1;
+		}
+		
+		fd = fopen(tpasswd, "w");
+		if (fd == NULL) {
+			fprintf(stderr, "Cannot open '%s' for write\n",
+				tpasswd);
+			remove(tmpname);
+			return -1;
+		}
+
+		fd2 = fopen(tmpname, "r");
+		if (fd2 == NULL) {
+			fprintf(stderr, "Cannot open '%s' for read\n",
+				tmpname);
+			remove(tmpname);
+			return -1;
+		}
+
+		put = 0;
+		do {
+			p = fgets( line, sizeof(line)-1, fd2);
+			if (p==NULL) break;
+			
+			pp = index( line, ':');
+			if (pp==NULL) continue;
+			
+			if ( strncmp( p, username, _MAX(strlen(username), (int)(pp-p)) ) == 0 ) {
+				put = 1;
+				fprintf(fd, "%s:%s:%u\n", username, cr, iindex);
+			} else {
+				fputs( line, fd);
+			}
+		} while(1);
+
+		if (put==0) {
+			fprintf(fd, "%s:%s:%u\n", username, cr, iindex);
+		}
 		free(cr);
+		
+		fclose(fd);
+		fclose(fd2);
+
+		remove(tmpname);
+
 	}
 
 
@@ -265,6 +416,7 @@ static int read_conf_values(MPI * g, MPI * n, char *str, int str_size)
 
 	/* read the generator */
 	len = strlen(p);
+	if (p[len-1]=='\n') len--;
 	tmp_size = _gnutls_sbase64_decode(p, len, &tmp);
 
 	if (tmp_size < 0) {
